using System;
using System.Collections;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;


namespace DrivingGame
{
	/// <summary>
	/// Summary description for Car_Delorean.
	/// </summary>
	public class Car_Delorean : Car
	{
		//-------------------------------------------------------------------------//
		public Car_Delorean(Ode.World w, Ode.Space s) : this(w, s, new Vector3())
		{
				
		}

		//-------------------------------------------------------------------------//
		public Car_Delorean(Ode.World w, Ode.Space s, Vector3 offset) : base(w, s, offset)
		{
			m_EnginePower   = 65;

			// Load and build the car model
			ArrayList ids; Geometry.LoadMultiOBJ("delorean.obj", out ids);
			BuildCarModel((int)ids[5], (int)ids[2], (int)ids[4], (int)ids[3], (int)ids[0]);
			SetCarTexture("Cars\\delorean.bmp");

			#region Setup Translation and Pivots for Wheels
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.X = 0.9f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Y = 0.358f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Z = 1.604f;
			m_WheelFrontRight.T.P.X = -m_WheelFrontRight.T.T.X;
			m_WheelFrontRight.T.P.Y = -m_WheelFrontRight.T.T.Y;
			m_WheelFrontRight.T.P.Z = -m_WheelFrontRight.T.T.Z;

			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.X = -0.9f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Y = 0.358f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Z = 1.604f;
			m_WheelFrontLeft.T.P.X = -m_WheelFrontLeft.T.T.X;
			m_WheelFrontLeft.T.P.Y = -m_WheelFrontLeft.T.T.Y;
			m_WheelFrontLeft.T.P.Z = -m_WheelFrontLeft.T.T.Z;

			((SceneNode)m_WheelBackRight.Children[0]).T.T.X = 1f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Y = 0.403f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Z = -1.384f;
			m_WheelBackRight.T.P.X = -m_WheelBackRight.T.T.X;
			m_WheelBackRight.T.P.Y = -m_WheelBackRight.T.T.Y;
			m_WheelBackRight.T.P.Z = -m_WheelBackRight.T.T.Z;

			((SceneNode)m_WheelBackLeft.Children[0]).T.T.X = -1f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Y = 0.403f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Z = -1.384f;
			m_WheelBackLeft.T.P.X = -m_WheelBackLeft.T.T.X;
			m_WheelBackLeft.T.P.Y = -m_WheelBackLeft.T.T.Y;
			m_WheelBackLeft.T.P.Z = -m_WheelBackLeft.T.T.Z;
			#endregion

			m_BodyPos     = new Vector3(0, 0.851f, 0.127f);
			m_BodySize    = new Vector3(2.353f, 1.224f, 5.360f);
			m_WheelRadius = 0.4f;
			m_COGYOffset  = -0.4f;
			BuildCarDynamics(m_BodyPos, m_BodySize, m_WheelRadius);

			BuildSNHeirarchy();
		}
	}
}
