using System;
using System.Collections;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Car_F1.
	/// </summary>
	public class Car_F1 : Car
	{

		private Vector3 YAxis = new Vector3(0, 1, 0);
		private SceneNode m_Head;

		//-------------------------------------------------------------------------//
		public Car_F1(Ode.World w, Ode.Space s) : this(w, s, new Vector3())
		{
			
		}

		//-------------------------------------------------------------------------//
		public Car_F1(Ode.World w, Ode.Space s, Vector3 offset) : base(w, s, offset)
		{
			this.NodeName = "F1 Car";

			m_EnginePower   = 100;
			m_TopSpeed      = 25;

			// Load and build the car model
			ArrayList ids; Geometry.LoadMultiOBJ("H-F1 Car-Move.obj", out ids);
			BuildCarModel((int)ids[0], (int)ids[5], (int)ids[4], (int)ids[1], (int)ids[3]);
			SetCarTexture("Cars\\f1car.BMP");

			#region Setup Translation and Pivots for Wheels
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.X = -1.017f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Y = 0.395f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Z = 1.407f;
			m_WheelFrontRight.T.P.X = -m_WheelFrontRight.T.T.X;
			m_WheelFrontRight.T.P.Y = -m_WheelFrontRight.T.T.Y;
			m_WheelFrontRight.T.P.Z = -m_WheelFrontRight.T.T.Z;

			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.X = 1.017f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Y = 0.395f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Z = 1.407f;
			m_WheelFrontLeft.T.P.X = -m_WheelFrontLeft.T.T.X;
			m_WheelFrontLeft.T.P.Y = -m_WheelFrontLeft.T.T.Y;
			m_WheelFrontLeft.T.P.Z = -m_WheelFrontLeft.T.T.Z;

			((SceneNode)m_WheelBackRight.Children[0]).T.T.X = -1.118f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Y = 0.395f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Z = -2.084f;
			m_WheelBackRight.T.P.X = -m_WheelBackRight.T.T.X;
			m_WheelBackRight.T.P.Y = -m_WheelBackRight.T.T.Y;
			m_WheelBackRight.T.P.Z = -m_WheelBackRight.T.T.Z;

			((SceneNode)m_WheelBackLeft.Children[0]).T.T.X = 1.118f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Y = 0.395f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Z = -2.084f;
			m_WheelBackLeft.T.P.X = -m_WheelBackLeft.T.T.X;
			m_WheelBackLeft.T.P.Y = -m_WheelBackLeft.T.T.Y;
			m_WheelBackLeft.T.P.Z = -m_WheelBackLeft.T.T.Z;
			#endregion

			m_BodyPos     = new Vector3(0, 0.485f, 0.047f);
			m_BodySize    = new Vector3(1.644f, 0.878f, 4.777f);
			m_WheelRadius = 0.375f;
			m_COGYOffset  = -0.485f;
			BuildCarDynamics(m_BodyPos, m_BodySize, m_WheelRadius);

			BuildSNHeirarchy();

			m_Head	= new SceneNode("Head", new LoadedPrimitive((int)ids[2]), new Transform());
			m_Head.T.T = new Vector3(0, 0.925f, 0.036f);
			m_Head.P.Texture = m_CarTexture;
			m_Body.AddChild(m_Head);
		}

		//-------------------------------------------------------------------------//
		public override void Animate(float timestep)
		{
			m_Head.T.RQ = new Quaternion(YAxis, -m_WheelJoints[0].Angle1 * 180.0f / (float)System.Math.PI);

			base.Animate (timestep);
		}

	}
}
