using System;
using System.Collections;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Car_Small.
	/// </summary>
	public class Car_Small : Car
	{
		//-------------------------------------------------------------------------//
		public Car_Small(Ode.World w, Ode.Space s) : this(w, s, new Vector3())
		{
					
		}

		//-------------------------------------------------------------------------//
		public Car_Small(Ode.World w, Ode.Space s, Vector3 offset) : base(w, s, offset)
		{
			// Load and build the car model
			ArrayList ids; Geometry.LoadMultiOBJ("H-Small Car White-Move.obj", out ids);
			BuildCarModel((int)ids[0], (int)ids[3], (int)ids[2], (int)ids[4], (int)ids[1]);
			SetCarTexture("Cars\\smallCar.bmp");

			#region Setup Translation and Pivots for Wheels
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.X = 0.627f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Y = 0.367f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Z = 0.979f;
			m_WheelFrontRight.T.P.X = -m_WheelFrontRight.T.T.X;
			m_WheelFrontRight.T.P.Y = -m_WheelFrontRight.T.T.Y;
			m_WheelFrontRight.T.P.Z = -m_WheelFrontRight.T.T.Z;

			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.X = -0.627f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Y = 0.367f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Z = 0.979f;
			m_WheelFrontLeft.T.P.X = -m_WheelFrontLeft.T.T.X;
			m_WheelFrontLeft.T.P.Y = -m_WheelFrontLeft.T.T.Y;
			m_WheelFrontLeft.T.P.Z = -m_WheelFrontLeft.T.T.Z;

			((SceneNode)m_WheelBackRight.Children[0]).T.T.X = 0.627f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Y = 0.367f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Z = -0.964f;
			m_WheelBackRight.T.P.X = -m_WheelBackRight.T.T.X;
			m_WheelBackRight.T.P.Y = -m_WheelBackRight.T.T.Y;
			m_WheelBackRight.T.P.Z = -m_WheelBackRight.T.T.Z;

			((SceneNode)m_WheelBackLeft.Children[0]).T.T.X = -0.627f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Y = 0.367f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Z = -0.964f;
			m_WheelBackLeft.T.P.X = -m_WheelBackLeft.T.T.X;
			m_WheelBackLeft.T.P.Y = -m_WheelBackLeft.T.T.Y;
			m_WheelBackLeft.T.P.Z = -m_WheelBackLeft.T.T.Z;
			#endregion

			m_BodyPos     = new Vector3(0, 0.815f, -0.015f);
			m_BodySize    = new Vector3(1.417f, 1.145f, 3.052f);
			m_WheelRadius = 0.275f;
			m_Density     = 25;
			m_COGYOffset  = -0.4f;
			BuildCarDynamics(m_BodyPos, m_BodySize, m_WheelRadius);

			BuildSNHeirarchy();

			SceneNode undercarriage = new SceneNode(new LoadedPrimitive((int)ids[5]), new Transform());
			m_Body.AddChild(undercarriage);
		}
	}
}
