using System;
using System.Collections;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Car_Stock.
	/// </summary>
	public class Car_Stock : Car
	{
		//-------------------------------------------------------------------------//
		public Car_Stock(Ode.World w, Ode.Space s) : this(w, s, new Vector3())
		{
				
		}

		//-------------------------------------------------------------------------//
		public Car_Stock(Ode.World w, Ode.Space s, Vector3 offset) : base(w, s, offset)
		{
			this.NodeName = "Stock Car";

			m_EnginePower   = 80;
			m_TopSpeed      = 20;

			// Load and build the car model
			ArrayList ids; Geometry.LoadMultiOBJ("H-Stock Car Blank-Move.obj", out ids);
			BuildCarModel((int)ids[0], (int)ids[5], (int)ids[3], (int)ids[4], (int)ids[2]);
			SetCarTexture("Cars\\stockCar2.bmp");	

			#region Setup Translation and Pivots for Wheels
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.X = -0.910f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Y = 0.325f;
			((SceneNode)m_WheelFrontRight.Children[0]).T.T.Z = 1.440f;
			m_WheelFrontRight.T.P.X = -m_WheelFrontRight.T.T.X;
			m_WheelFrontRight.T.P.Y = -m_WheelFrontRight.T.T.Y;
			m_WheelFrontRight.T.P.Z = -m_WheelFrontRight.T.T.Z;

			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.X = 0.910f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Y = 0.325f;
			((SceneNode)m_WheelFrontLeft.Children[0]).T.T.Z = 1.440f;
			m_WheelFrontLeft.T.P.X = -m_WheelFrontLeft.T.T.X;
			m_WheelFrontLeft.T.P.Y = -m_WheelFrontLeft.T.T.Y;
			m_WheelFrontLeft.T.P.Z = -m_WheelFrontLeft.T.T.Z;

			((SceneNode)m_WheelBackRight.Children[0]).T.T.X = -0.924f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Y = 0.330f;
			((SceneNode)m_WheelBackRight.Children[0]).T.T.Z = -1.529f;
			m_WheelBackRight.T.P.X = -m_WheelBackRight.T.T.X;
			m_WheelBackRight.T.P.Y = -m_WheelBackRight.T.T.Y;
			m_WheelBackRight.T.P.Z = -m_WheelBackRight.T.T.Z;

			((SceneNode)m_WheelBackLeft.Children[0]).T.T.X = 0.924f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Y = 0.330f;
			((SceneNode)m_WheelBackLeft.Children[0]).T.T.Z = -1.529f;
			m_WheelBackLeft.T.P.X = -m_WheelBackLeft.T.T.X;
			m_WheelBackLeft.T.P.Y = -m_WheelBackLeft.T.T.Y;
			m_WheelBackLeft.T.P.Z = -m_WheelBackLeft.T.T.Z;
			#endregion

			m_BodyPos     = new Vector3(0, 0.745f, 0.109f);
			m_BodySize    = new Vector3(1.967f, 1.376f, 5.452f);
			m_WheelRadius = 0.4f;
			m_COGYOffset  = -0.67f;
			BuildCarDynamics(m_BodyPos, m_BodySize, m_WheelRadius);

			BuildSNHeirarchy();

			SceneNode uc = new SceneNode("Undercarriage", new LoadedPrimitive((int)ids[1]), new Transform());
			uc.P.Texture = m_CarTexture;
			m_Body.AddChild(uc);
		}
	}
}
