using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for LoadingForm.
	/// </summary>
	public class LoadingForm : System.Windows.Forms.Form
	{
		//-----------------------------------//
		private static LoadingForm m_Form       = null;
		private static Thread      m_Thread     = null;
		private static Queue       m_QueuedJobs = new Queue(3);
		//-----------------------------------//
		public static LoadingForm LoadForm
		{
			get { return m_Form; }
		}
		//-----------------------------------//
		
		//-----------------------------------//
		private double    m_dblOpacityIncrement = .075;
		private double    m_dblOpacityDecrement = .1;
		private const int TIMER_INTERVAL = 50;
		private string    m_Status;
		//-----------------------------------//

		//-----------------------------------//
		private int       m_JobCount = 7;
		private int       m_JobsDone = -1;
		private Rectangle m_Progress;
		//-----------------------------------//


		private System.Windows.Forms.Label       label1;
		private System.Windows.Forms.Timer       m_FadeTimer;
		private System.Windows.Forms.Panel       Panel_Border;
		private System.Windows.Forms.Panel       Panel_Progress;
		private System.Windows.Forms.PictureBox  pictureBox1;
		private System.Windows.Forms.Label label2;
		private System.ComponentModel.IContainer components;
		

		//-------------------------------------------------------------------------//
		public LoadingForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.Opacity = 0;
			m_FadeTimer.Interval = TIMER_INTERVAL;
			m_FadeTimer.Start();
		}

		//-------------------------------------------------------------------------//
		public static void SetCurrentJob(string newStatus)
		{
			m_QueuedJobs.Enqueue(newStatus);
		}

		//-------------------------------------------------------------------------//
		public static void ShowSplashScreen()
		{
			if (m_Form != null) return;

			m_Thread = new Thread(new ThreadStart(LoadingForm.ShowForm));
			m_Thread.IsBackground = true;
			m_Thread.ApartmentState = ApartmentState.STA;
			m_Thread.Start();
		}

		//-------------------------------------------------------------------------//
		// A static entry point to launch SplashScreen.
		private static void ShowForm()
		{
			m_Form = new LoadingForm();
			Application.Run(m_Form);
		}

		//-------------------------------------------------------------------------//
		// A static method to close the SplashScreen
		public static void CloseForm()
		{
			if( m_Form != null )
			{
				// Make it start going away.
				m_Form.m_dblOpacityIncrement = -m_Form.m_dblOpacityDecrement;
			}

			//m_Thread = null;  // we do not need these any more.
			m_Form   = null;
		}

		//-------------------------------------------------------------------------//
		private void m_FadeTimer_Tick(object sender, System.EventArgs e)
		{
			// Clear queue
			while (m_QueuedJobs.Count > 0)
			{
				m_JobsDone++;
				m_Status = (string)m_QueuedJobs.Dequeue();
			}
			// Update Text Label
			label1.Text = m_Status;

			// Update Progress Bar
			int width = (int)Math.Floor(Panel_Progress.ClientRectangle.Width 
				* ((double)m_JobsDone / (double)m_JobCount));
			int height = Panel_Progress.ClientRectangle.Height;
			int x = Panel_Progress.ClientRectangle.X;
			int y = Panel_Progress.ClientRectangle.Y;
			if( width > 0 && height > 0 )
			{
				m_Progress = new Rectangle(x, y, width, height);
				Panel_Progress.Invalidate(m_Progress);
			}

			// Fade
			if( m_dblOpacityIncrement > 0 )
			{
				if( this.Opacity < 1 )
					this.Opacity += m_dblOpacityIncrement;
			}
			else
			{
				if( this.Opacity > 0 )
					this.Opacity += m_dblOpacityIncrement;
				else
				{
					this.Close();
					if (m_Thread != null)
					{
						m_Thread.Abort();
						m_Thread = null;
					}
				}
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(LoadingForm));
			this.label1 = new System.Windows.Forms.Label();
			this.m_FadeTimer = new System.Windows.Forms.Timer(this.components);
			this.Panel_Border = new System.Windows.Forms.Panel();
			this.label2 = new System.Windows.Forms.Label();
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.Panel_Progress = new System.Windows.Forms.Panel();
			this.Panel_Border.SuspendLayout();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 48);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(192, 16);
			this.label1.TabIndex = 0;
			this.label1.Text = "LOADING...";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// m_FadeTimer
			// 
			this.m_FadeTimer.Tick += new System.EventHandler(this.m_FadeTimer_Tick);
			// 
			// Panel_Border
			// 
			this.Panel_Border.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.Panel_Border.BackColor = System.Drawing.Color.White;
			this.Panel_Border.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Panel_Border.Controls.Add(this.label2);
			this.Panel_Border.Controls.Add(this.pictureBox1);
			this.Panel_Border.Controls.Add(this.Panel_Progress);
			this.Panel_Border.Controls.Add(this.label1);
			this.Panel_Border.Location = new System.Drawing.Point(0, 0);
			this.Panel_Border.Name = "Panel_Border";
			this.Panel_Border.Size = new System.Drawing.Size(264, 104);
			this.Panel_Border.TabIndex = 1;
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Frutiger Linotype", 12F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label2.Location = new System.Drawing.Point(8, 8);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(152, 32);
			this.label2.TabIndex = 3;
			this.label2.Text = "Please Wait...";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// pictureBox1
			// 
			this.pictureBox1.BackColor = System.Drawing.Color.Transparent;
			this.pictureBox1.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox1.Image")));
			this.pictureBox1.Location = new System.Drawing.Point(208, 8);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(48, 48);
			this.pictureBox1.TabIndex = 2;
			this.pictureBox1.TabStop = false;
			// 
			// Panel_Progress
			// 
			this.Panel_Progress.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Panel_Progress.Location = new System.Drawing.Point(8, 72);
			this.Panel_Progress.Name = "Panel_Progress";
			this.Panel_Progress.Size = new System.Drawing.Size(248, 24);
			this.Panel_Progress.TabIndex = 1;
			this.Panel_Progress.Paint += new System.Windows.Forms.PaintEventHandler(this.Panel_Progress_Paint);
			// 
			// LoadingForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(264, 104);
			this.Controls.Add(this.Panel_Border);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
			this.Name = "LoadingForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.TopMost = true;
			this.Panel_Border.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void Panel_Progress_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if( e.ClipRectangle.Width > 0 && m_JobsDone > 0 )
			{
				LinearGradientBrush brBackground = 
					new LinearGradientBrush(m_Progress, 
					Color.FromArgb(50, 50, 200),
					Color.FromArgb(150, 150, 255), 
					LinearGradientMode.Horizontal);
				e.Graphics.FillRectangle(brBackground, m_Progress);

				brBackground.Dispose();
			}
		}

	}
}
