using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;


namespace DrivingGame
{
	/// <summary>
	/// Main program form.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		#region High Performance Counter DllImports
		[System.Runtime.InteropServices.DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceCounter(out long lpPerformanceCount);
		[System.Runtime.InteropServices.DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceFrequency(out long lpFrequency);
		#endregion

		//-----------------------------------//
		private long LastPaint;              // Last time we painted
		private long PerfCountFreq;          // Performance counter frequency
		private long LastFPS;                // Last time we output FPS
		private int  FrameCount;             // Number of frames since LastFPS
		//-----------------------------------//

		//-----------------------------------//
		private WorldModel World;            // The World Model
		//-----------------------------------//

		//-----------------------------------// Main Viewport/UIWindow
		private JPGL.SWF.UIWindow UIWindow_Main;
		private Viewport_Main     View_Main;
		//-----------------------------------//

		//-----------------------------------// Child Window Handles
		private OverviewForm     OverviewWindow = null;
		//-----------------------------------//

		private System.Windows.Forms.StatusBar      StatusBar;
		private System.Windows.Forms.StatusBarPanel StatusBarPanel_Echo;
		private System.Windows.Forms.StatusBarPanel StatusBarPanel_FPS;
		private System.Windows.Forms.MainMenu       MainMenu;
		private System.Windows.Forms.MenuItem       MenuItem_Reset;
		private System.Windows.Forms.MenuItem       MenuItem_Quit;
		private System.Windows.Forms.MenuItem       MenuItem_ShowOverview;
		private System.Windows.Forms.MenuItem       MenuItem_View;
		private System.Windows.Forms.MenuItem       MenuItem_FrameLimiter;
		private System.Windows.Forms.MenuItem       MenuItem_Menu;
		private System.Windows.Forms.MenuItem       MenuItem_Seperator4;
		private System.Windows.Forms.MenuItem MenuItem_Seperator5;
		private System.Windows.Forms.MenuItem MenuItem_RunSim;
		private System.Windows.Forms.MenuItem MenuItem_ViewType;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeBehind;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeFarBehind;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeFront;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeAbove;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeFree;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeDriveBy;
		private System.Windows.Forms.MenuItem MenuItem_ShowHUD;
		private System.Windows.Forms.MenuItem MenuItem_SelCar;
		private System.Windows.Forms.MenuItem MenuItem_Help;
		private System.Windows.Forms.MenuItem MenuItem_ViewTypeDriver;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem MenuItem_Fireworks;
		private System.ComponentModel.IContainer components;

		/// <summary>
		/// Constructor
		/// </summary>
		public MainForm()
		{
			LoadingForm.SetCurrentJob("Initializing Main Form");
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Create the model
			LoadingForm.SetCurrentJob("Loading World");
			World = new WorldModel();

			// Create the views
			LoadingForm.SetCurrentJob("Initializing Main View");
			View_Main = new Viewport_Main(World);
			LoadingForm.SetCurrentJob("Attaching Main View");
			UIWindow_Main.AttachViewport(View_Main);
			
			// Status bar echo
			LoadingForm.SetCurrentJob("Initialzing Status Bar");
			JPGL.SWF.UIUtility.SetEchoPanel(StatusBarPanel_Echo);

			// Timer Variables
			LoadingForm.SetCurrentJob("Initializing Timers");
			QueryPerformanceFrequency(out PerfCountFreq);
			QueryPerformanceCounter(out LastPaint);
			QueryPerformanceCounter(out LastFPS);
			FrameCount = 0;

			LoadingForm.SetCurrentJob("Building Fonts");
			JPGL.OpenGL.GraphicsSystem.BuildFonts("fonts.png");

			// When one of the following events happens, we want
			// to update our timer. These changes both cause 
			// Application.DoEvents to take a very long time,
			// which in turn causes the timestep to be large
			// for an iteration and the Dynamics Engine goes
			// nuts with a large timestep.
			this.Resize += new EventHandler(MainForm_Change);
			this.Move   += new EventHandler(MainForm_Change);
		}

		/// <summary>
		/// Update timer
		/// </summary>
		private void MainForm_Change(object sender, EventArgs e)
		{
			QueryPerformanceCounter(out LastPaint);
			QueryPerformanceCounter(out LastFPS);
			FrameCount = 0;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.UIWindow_Main = new JPGL.SWF.UIWindow();
			this.StatusBar = new System.Windows.Forms.StatusBar();
			this.StatusBarPanel_Echo = new System.Windows.Forms.StatusBarPanel();
			this.StatusBarPanel_FPS = new System.Windows.Forms.StatusBarPanel();
			this.MainMenu = new System.Windows.Forms.MainMenu();
			this.MenuItem_Menu = new System.Windows.Forms.MenuItem();
			this.MenuItem_RunSim = new System.Windows.Forms.MenuItem();
			this.MenuItem_Reset = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.MenuItem_SelCar = new System.Windows.Forms.MenuItem();
			this.MenuItem_Fireworks = new System.Windows.Forms.MenuItem();
			this.MenuItem_FrameLimiter = new System.Windows.Forms.MenuItem();
			this.MenuItem_Seperator4 = new System.Windows.Forms.MenuItem();
			this.MenuItem_Quit = new System.Windows.Forms.MenuItem();
			this.MenuItem_View = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewType = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeBehind = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeFarBehind = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeFront = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeAbove = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeFree = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeDriveBy = new System.Windows.Forms.MenuItem();
			this.MenuItem_ViewTypeDriver = new System.Windows.Forms.MenuItem();
			this.MenuItem_ShowHUD = new System.Windows.Forms.MenuItem();
			this.MenuItem_Seperator5 = new System.Windows.Forms.MenuItem();
			this.MenuItem_ShowOverview = new System.Windows.Forms.MenuItem();
			this.MenuItem_Help = new System.Windows.Forms.MenuItem();
			((System.ComponentModel.ISupportInitialize)(this.StatusBarPanel_Echo)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.StatusBarPanel_FPS)).BeginInit();
			this.SuspendLayout();
			// 
			// UIWindow_Main
			// 
			this.UIWindow_Main.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.UIWindow_Main.Location = new System.Drawing.Point(0, 0);
			this.UIWindow_Main.Name = "UIWindow_Main";
			this.UIWindow_Main.Size = new System.Drawing.Size(512, 504);
			this.UIWindow_Main.TabIndex = 0;
			// 
			// StatusBar
			// 
			this.StatusBar.Location = new System.Drawing.Point(0, 507);
			this.StatusBar.Name = "StatusBar";
			this.StatusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.StatusBarPanel_Echo,
																						 this.StatusBarPanel_FPS});
			this.StatusBar.ShowPanels = true;
			this.StatusBar.Size = new System.Drawing.Size(512, 22);
			this.StatusBar.TabIndex = 1;
			// 
			// StatusBarPanel_Echo
			// 
			this.StatusBarPanel_Echo.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.StatusBarPanel_Echo.Width = 446;
			// 
			// StatusBarPanel_FPS
			// 
			this.StatusBarPanel_FPS.Alignment = System.Windows.Forms.HorizontalAlignment.Center;
			this.StatusBarPanel_FPS.Text = "FPS: -";
			this.StatusBarPanel_FPS.Width = 50;
			// 
			// MainMenu
			// 
			this.MainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.MenuItem_Menu,
																					 this.MenuItem_View,
																					 this.MenuItem_Help});
			// 
			// MenuItem_Menu
			// 
			this.MenuItem_Menu.Index = 0;
			this.MenuItem_Menu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.MenuItem_RunSim,
																						  this.MenuItem_Reset,
																						  this.menuItem1,
																						  this.MenuItem_SelCar,
																						  this.MenuItem_Fireworks,
																						  this.MenuItem_FrameLimiter,
																						  this.MenuItem_Seperator4,
																						  this.MenuItem_Quit});
			this.MenuItem_Menu.Text = "Menu";
			// 
			// MenuItem_RunSim
			// 
			this.MenuItem_RunSim.Index = 0;
			this.MenuItem_RunSim.Text = "Pause";
			this.MenuItem_RunSim.Click += new System.EventHandler(this.MenuItem_RunSim_Click);
			// 
			// MenuItem_Reset
			// 
			this.MenuItem_Reset.Index = 1;
			this.MenuItem_Reset.Text = "Reset";
			this.MenuItem_Reset.Click += new System.EventHandler(this.MenuItem_Reset_Click);
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 2;
			this.menuItem1.Text = "-";
			// 
			// MenuItem_SelCar
			// 
			this.MenuItem_SelCar.Index = 3;
			this.MenuItem_SelCar.Text = "Change Vehicle";
			this.MenuItem_SelCar.Click += new System.EventHandler(this.MenuItem_SelCar_Click);
			// 
			// MenuItem_Fireworks
			// 
			this.MenuItem_Fireworks.Index = 4;
			this.MenuItem_Fireworks.Text = "Shoot Fireworks";
			this.MenuItem_Fireworks.Click += new System.EventHandler(this.MenuItem_Fireworks_Click);
			// 
			// MenuItem_FrameLimiter
			// 
			this.MenuItem_FrameLimiter.Checked = true;
			this.MenuItem_FrameLimiter.Index = 5;
			this.MenuItem_FrameLimiter.Text = "Enable Frame Limiter";
			this.MenuItem_FrameLimiter.Click += new System.EventHandler(this.MenuItem_FrameLimiter_Click);
			// 
			// MenuItem_Seperator4
			// 
			this.MenuItem_Seperator4.Index = 6;
			this.MenuItem_Seperator4.Text = "-";
			// 
			// MenuItem_Quit
			// 
			this.MenuItem_Quit.Index = 7;
			this.MenuItem_Quit.Text = "Quit";
			this.MenuItem_Quit.Click += new System.EventHandler(this.MenuItem_Quit_Click);
			// 
			// MenuItem_View
			// 
			this.MenuItem_View.Index = 1;
			this.MenuItem_View.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.MenuItem_ViewType,
																						  this.MenuItem_ShowHUD,
																						  this.MenuItem_Seperator5,
																						  this.MenuItem_ShowOverview});
			this.MenuItem_View.Text = "View";
			// 
			// MenuItem_ViewType
			// 
			this.MenuItem_ViewType.Index = 0;
			this.MenuItem_ViewType.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							  this.MenuItem_ViewTypeBehind,
																							  this.MenuItem_ViewTypeFarBehind,
																							  this.MenuItem_ViewTypeFront,
																							  this.MenuItem_ViewTypeAbove,
																							  this.MenuItem_ViewTypeFree,
																							  this.MenuItem_ViewTypeDriveBy,
																							  this.MenuItem_ViewTypeDriver});
			this.MenuItem_ViewType.Text = "Camera View";
			// 
			// MenuItem_ViewTypeBehind
			// 
			this.MenuItem_ViewTypeBehind.Index = 0;
			this.MenuItem_ViewTypeBehind.Text = "Behind";
			this.MenuItem_ViewTypeBehind.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeFarBehind
			// 
			this.MenuItem_ViewTypeFarBehind.Index = 1;
			this.MenuItem_ViewTypeFarBehind.Text = "Far Behind";
			this.MenuItem_ViewTypeFarBehind.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeFront
			// 
			this.MenuItem_ViewTypeFront.Index = 2;
			this.MenuItem_ViewTypeFront.Text = "Front";
			this.MenuItem_ViewTypeFront.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeAbove
			// 
			this.MenuItem_ViewTypeAbove.Index = 3;
			this.MenuItem_ViewTypeAbove.Text = "Above";
			this.MenuItem_ViewTypeAbove.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeFree
			// 
			this.MenuItem_ViewTypeFree.Index = 4;
			this.MenuItem_ViewTypeFree.Text = "Free";
			this.MenuItem_ViewTypeFree.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeDriveBy
			// 
			this.MenuItem_ViewTypeDriveBy.Index = 5;
			this.MenuItem_ViewTypeDriveBy.Text = "Drive By";
			this.MenuItem_ViewTypeDriveBy.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ViewTypeDriver
			// 
			this.MenuItem_ViewTypeDriver.Index = 6;
			this.MenuItem_ViewTypeDriver.Text = "Driver";
			this.MenuItem_ViewTypeDriver.Click += new System.EventHandler(this.MenuItem_ViewTypes_Click);
			// 
			// MenuItem_ShowHUD
			// 
			this.MenuItem_ShowHUD.Checked = true;
			this.MenuItem_ShowHUD.Index = 1;
			this.MenuItem_ShowHUD.Text = "Show Speedometer";
			this.MenuItem_ShowHUD.Click += new System.EventHandler(this.MenuItem_ShowHUD_Click);
			// 
			// MenuItem_Seperator5
			// 
			this.MenuItem_Seperator5.Index = 2;
			this.MenuItem_Seperator5.Text = "-";
			// 
			// MenuItem_ShowOverview
			// 
			this.MenuItem_ShowOverview.Index = 3;
			this.MenuItem_ShowOverview.Text = "Overview Window";
			this.MenuItem_ShowOverview.Click += new System.EventHandler(this.menuItem2_Click);
			// 
			// MenuItem_Help
			// 
			this.MenuItem_Help.Index = 2;
			this.MenuItem_Help.Text = "Help";
			this.MenuItem_Help.Click += new System.EventHandler(this.MenuItem_Help_Click);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(512, 529);
			this.Controls.Add(this.StatusBar);
			this.Controls.Add(this.UIWindow_Main);
			this.KeyPreview = true;
			this.Menu = this.MainMenu;
			this.Name = "MainForm";
			this.Text = "Crash!";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.MainForm_KeyDown);
			this.KeyUp += new System.Windows.Forms.KeyEventHandler(this.MainForm_KeyUp);
			((System.ComponentModel.ISupportInitialize)(this.StatusBarPanel_Echo)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.StatusBarPanel_FPS)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			LoadingForm.ShowSplashScreen();

			using (MainForm frm = new MainForm())
			{
				LoadingForm.SetCurrentJob("Done!");
				LoadingForm.LoadForm.Owner = frm;

				// Hide the splash screen and show the form
				LoadingForm.CloseForm();
				frm.Show();

				// While the form is still valid, render and process messages
				while(frm.Created)
				{
					frm.Render();
					Application.DoEvents();
				}
			}
		}

		/// <summary>
		/// Non-Timer based rendering function. Calculates the time elapsed since it
		/// was last called, then updates the Model based on that elapsed time, and
		/// renders the viewport(s). A FPS count is calculated and output once a second.
		/// Something interesting to note is that the static Main() function is calling
		/// DoEvents() - which processes waiting S.W.F events. Thus, any activity going
		/// on in the application using S.W.F stuff will get priority over Render().
		/// </summary>
		public void Render()
		{
			// Get elapsed time since last paint (in seconds)
			long stop; QueryPerformanceCounter(out stop);
			float elapsed = (float)(stop-LastPaint) / (float)PerfCountFreq;

			// Lets not go all out here. We dont really need to draw anything
			// faster than 60 FPS, so if on 1/60th of a second (roughly) has
			// elapsed, lets take a nap for a bit. Note that Sleep(int) will 
			// throw an exception if given a negative number.
			if (MenuItem_FrameLimiter.Checked && elapsed < 0.016f)
			{
				int ms = 16 - (int)(elapsed * 1000f);
				if (ms > 0)
					System.Threading.Thread.Sleep(ms);
				
				QueryPerformanceCounter(out stop);
				elapsed = (float)(stop-LastPaint) / (float)PerfCountFreq;
			}

			// Update the model, but only if fewer than 2 seconds have elapsed
			// this allows for opening of menus to take focus from the drawing
			// of the UI, yet the world will not continue to be updated while 
			// the menu is open. Of course, this is a bit of a problem if the 
			// machine cant render a frame in under 2 seconds...
			if (elapsed < 2.0f)
				World.UpdateWorld(elapsed, View_Main);

			// Draw the views; whichever ones are available.
			UIWindow_Main.DrawViewport();

			if (OverviewWindow != null && !OverviewWindow.Disposing && OverviewWindow.Visible)
				OverviewWindow.UIWindow.DrawViewport();

			// Update last paint time
			LastPaint = stop;

			// If 1 second has elapsed, output FrameCount
			if (((float)(stop-LastFPS) / (float)PerfCountFreq) >= 1)
			{
				float fps = (float)FrameCount / ((float)(stop-LastFPS) / (float)PerfCountFreq);
				StatusBarPanel_FPS.Text = "FPS: " + ((int)fps);

				FrameCount = 0;
				LastFPS = stop;
			}

			// Increment frame counter
			FrameCount++;
		}

		/// <summary>
		/// Show Overview window
		/// </summary>
		private void menuItem2_Click(object sender, System.EventArgs e)
		{
			if (OverviewWindow != null && !OverviewWindow.Disposing && OverviewWindow.Visible)
				OverviewWindow.Activate();
			else
			{
				OverviewWindow = new OverviewForm(World, View_Main);
				OverviewWindow.Show();
			}
		}

		/// <summary>
		/// Reset everything
		/// </summary>
		private void MenuItem_Reset_Click(object sender, System.EventArgs e)
		{
			World.ResetWorld();

			// Reset viewports
			View_Main.Reset();
			if (OverviewWindow != null && !OverviewWindow.Disposing && OverviewWindow.Visible)
				OverviewWindow.Viewport.Reset();

			MenuItem_Fireworks.Checked = false;
		}

		/// <summary>
		/// Quit the program
		/// </summary>
		private void MenuItem_Quit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		/// <summary>
		/// Set check on this MenuItem only, thank you S.W.F.
		/// </summary>
		private void MenuItem_FrameLimiter_Click(object sender, System.EventArgs e)
		{
			MenuItem_FrameLimiter.Checked = !MenuItem_FrameLimiter.Checked;
		}

		private void MenuItem_RunSim_Click(object sender, System.EventArgs e)
		{
			MenuItem_RunSim.Checked = !MenuItem_RunSim.Checked;
			World.SimRunning = !MenuItem_RunSim.Checked;
		}

		private void MainForm_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			View_Main.KeyDown(e);
		}

		private void MainForm_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			View_Main.KeyUp(e);
		}

		private void MenuItem_ViewTypes_Click(object sender, System.EventArgs e)
		{
			if (((MenuItem)sender).Checked) return;

			// Uncheck other types, check sender
			foreach (MenuItem m in MenuItem_ViewType.MenuItems)
				m.Checked = false;
			((MenuItem)sender).Checked = true;

			// Update view
			if      (MenuItem_ViewTypeBehind.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.BEHIND;
			else if (MenuItem_ViewTypeFarBehind.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.FARBEHIND;
			else if (MenuItem_ViewTypeFront.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.FRONT;
			else if (MenuItem_ViewTypeAbove.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.OVERHEAD;
			else if (MenuItem_ViewTypeFree.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.FREE;
			else if (MenuItem_ViewTypeDriver.Checked) 
				View_Main.CameraView = Viewport_Main.CameraViewType.DRIVER;
			
		}

		private void MenuItem_ShowHUD_Click(object sender, System.EventArgs e)
		{
			MenuItem_ShowHUD.Checked = !MenuItem_ShowHUD.Checked;
			View_Main.ShowHUD = MenuItem_ShowHUD.Checked;
		}

		private void MenuItem_SelCar_Click(object sender, System.EventArgs e)
		{
			CarSelectionForm CarSelWindow = new CarSelectionForm(World);
			CarSelWindow.Show();
		}

		private void MenuItem_Help_Click(object sender, System.EventArgs e)
		{
			HelpForm f = new HelpForm();
			f.Show();
		}

		private void MenuItem_Fireworks_Click(object sender, System.EventArgs e)
		{
			MenuItem_Fireworks.Checked = !MenuItem_Fireworks.Checked;
			World.m_PartSysOn = MenuItem_Fireworks.Checked;
		}
	}
}
