using System;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Box.
	/// </summary>
	public class Box : SceneNode
	{
		private static Texture CrateTexture = JPGL.OpenGL.GraphicsSystem.LoadTexture("crate.bmp");
		private static Cube m_Cube = new Cube();

		//-----------------------------------//
		private Ode.Body m_OdeBody;
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public Box(float x, float y, float z, Ode.World w, Ode.Space s) : base(m_Cube, new Transform())
		{
			T.T.X = x;
			T.T.Y = y;
			T.T.Z = z;

			P.Texture = CrateTexture;

			// Setup Dynamics
			m_OdeBody = new Ode.Body(w);
			m_OdeBody.Position = new Ode.Vector3(T.T.X, T.T.Y, T.T.Z);  
			m_OdeBody.Geoms.Add(new Ode.BoxGeom(s, 1, 1, 1));

			Ode.Mass m = new Ode.Mass();
			m.SetBox(2, 1, 1, 1);
			m_OdeBody.SetMass(m);
		}

		//-------------------------------------------------------------------------//
		public void Reset(float x, float y, float z)
		{
			m_OdeBody.Position = new Ode.Vector3(x, y, z);
			m_OdeBody.Quaternion = new Ode.Quaternion();
		}

		//-------------------------------------------------------------------------//
		public void Animate(float timestep)
		{
			T.T.X = m_OdeBody.Position.X;
			T.T.Y = m_OdeBody.Position.Y;
			T.T.Z = m_OdeBody.Position.Z;
			T.RQ = new Quaternion(
				m_OdeBody.Quaternion.W, m_OdeBody.Quaternion.X,
				m_OdeBody.Quaternion.Y, m_OdeBody.Quaternion.Z);
		}
	}
}
