using System;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Fence.
	/// </summary>
	public class Fence : SceneNode
	{
		//-----------------------------------//
		private static Texture m_FenceTex  = JPGL.OpenGL.GraphicsSystem.LoadTexture("woodfence.jpg");
		private static int     m_FenceGeom = Geometry.LoadOBJ("WoodFence.obj");
		//-----------------------------------//

		private Ode.Body m_OdeBody;


		//-------------------------------------------------------------------------//
		public Fence(float x, float y, float z, float rot, Ode.World w, Ode.Space s) : base(new LoadedPrimitive(m_FenceGeom), new Transform())
		{
			T.T.X = x; T.T.Y = y; T.T.Z = z;
			T.P.X = -0.997f;
			T.P.Y = 1.000f;
			T.P.Z = 0.096f;

			P.Texture = m_FenceTex;

			m_OdeBody = new Ode.Body(w);
			m_OdeBody.Geoms.Add(new Ode.BoxGeom(s, 1.995f, 2.000f, 0.327f));
			m_OdeBody.Position = new Ode.Vector3(T.P.X+x, T.P.Y+y, T.P.Z+z);
			if (rot != 0f)
			{
				Quaternion q = new Quaternion(new Vector3(0, 1, 0), rot);
				Ode.Quaternion qode = new Ode.Quaternion();
				qode.X = q.X; qode.Y = q.Y; qode.Z = q.Z; qode.W = q.W;
				m_OdeBody.Quaternion = qode;
			}
		}

		//-------------------------------------------------------------------------//
		public void Reset(float x, float y, float z, float rot)
		{
			m_OdeBody.Position = new Ode.Vector3(T.P.X+x, T.P.Y+y, T.P.Z+z);

			if (rot != 0f)
			{
				Quaternion q = new Quaternion(new Vector3(0, 1, 0), rot);
				Ode.Quaternion qode = new Ode.Quaternion();
				qode.X = q.X; qode.Y = q.Y; qode.Z = q.Z; qode.W = q.W;
				m_OdeBody.Quaternion = qode;
			}
			else
			{
				m_OdeBody.Quaternion = new Ode.Quaternion();
			}
		}

		//-------------------------------------------------------------------------//
		public void Animate(float timestep)
		{
			T.T.X = m_OdeBody.Position.X-T.P.X;
			T.T.Y = m_OdeBody.Position.Y-T.P.Y;
			T.T.Z = m_OdeBody.Position.Z-T.P.Z;

			T.RQ.X = m_OdeBody.Quaternion.X;
			T.RQ.Y = m_OdeBody.Quaternion.Y;
			T.RQ.Z = m_OdeBody.Quaternion.Z;
			T.RQ.W = m_OdeBody.Quaternion.W;

		}
	}
}
