using System;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	#region GroundGeometry Class
	public class GroundGeometry : DynamicGeometry
	{
		private int m_VertexCount;
		public override int VertexCount
		{
			get
			{
				return m_VertexCount;
			}
			set
			{
				m_VertexCount = value;
			}
		}

		public GroundGeometry() : base(PrimitiveRenderer.QUADS)
		{
			this.AutoCache = false;
			m_VertexCount = 0;
		}
	}
	#endregion
 
	#region GroundPrimitive Class
	public class GroundPrimitive : FillablePrimitive
	{
		//-----------------------------------// 
		private GroundGeometry m_Geometry;
		private Transform      m_Trans;
		//-----------------------------------// 
		
		//-----------------------------------// 
		public override bool Defined     
		{              
			get
			{
				return (m_Geometry != null);
			}
		}
		//-----------------------------------//

		//-----------------------------------//
		public GroundGeometry Geometry
		{
			get { return m_Geometry;  }
			set { m_Geometry = value; }
		}
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public GroundPrimitive() : base()
		{
			m_Trans = new Transform();
		}

		//-------------------------------------------------------------------------//
		public override Geometry GetGeometry()
		{
			return m_Geometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return m_Trans;
		}

		//-------------------------------------------------------------------------//
		public override JPGL.Common.Colliders.ColliderList GetColliders()
		{
			return new JPGL.Common.Colliders.ColliderList();
		}
	}
	#endregion

	/// <summary>
	/// Summary description for Ground.
	/// </summary>
	public class Ground : SceneNode, IDisposable
	{
		//-----------------------------------//
		private int       m_Rows,m_Cols;
		private int[,]    m_TileTable;
		private Texture[] m_TextureTable;
		//-----------------------------------//

		//-----------------------------------//
		private int           m_CacheId;
		private Ode.PlaneGeom m_OdePlane;
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public Ground(Ode.Space s) : base(null, new Transform())
		{
			m_OdePlane = new Ode.PlaneGeom(s, 0, 1, 0, 0);

			T.T.X = 2.5f;
			T.T.Z = 2.5f;
			T.S.X = 5;
			T.S.Z = 5;

			// Load the textures
			m_TextureTable = new Texture[17];
			m_TextureTable[0] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\grndjustsand.jpg");
			m_TextureTable[1] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\grndroad1left.jpg");
			m_TextureTable[2] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\grndroad1mid2.jpg");
			m_TextureTable[3] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\grndroad1right.jpg");
			m_TextureTable[4] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\grndroad1corner2.jpg");
			m_TextureTable[5] = new Texture(m_TextureTable[1]); m_TextureTable[5].Rotation = -90;
			m_TextureTable[6] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\scrub1side.jpg");
			m_TextureTable[7] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\scrub1.jpg");
			m_TextureTable[8] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\scrub1inlet.jpg");
			m_TextureTable[9] = JPGL.OpenGL.GraphicsSystem.LoadTexture("GroundTiles\\scrub1peninsula.jpg");
			m_TextureTable[10] = new Texture(m_TextureTable[6]); m_TextureTable[10].Rotation = -90;
			m_TextureTable[11] = new Texture(m_TextureTable[6]); m_TextureTable[11].Rotation = 180;
			m_TextureTable[12] = new Texture(m_TextureTable[9]); m_TextureTable[12].Rotation = -90;
			m_TextureTable[13] = new Texture(m_TextureTable[4]); m_TextureTable[13].Rotation = -90;
			m_TextureTable[14] = new Texture(m_TextureTable[9]);  m_TextureTable[14].Rotation = 90;
			m_TextureTable[15] = new Texture(m_TextureTable[6]);  m_TextureTable[15].Rotation = 90;
			m_TextureTable[16] = new Texture(m_TextureTable[9]);  m_TextureTable[16].Rotation = 180;

			for (int i=0; i<m_TextureTable.Length; i++)
				m_TextureTable[i].SetTextureTiled();

			m_Rows = 30;
			m_Cols = 30;

			// Yes, this could, would, should really be in a file...
			m_TileTable = new int[,]
				{
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1,11,11,11,12, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 7, 7, 7,10, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 7, 7, 7,10, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 1, 7, 7, 7,10, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 4, 5, 5, 5, 5, 5, 5, 5, 5, 5,13, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,16},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,16, 8},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,16,11,12, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7,10, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,14, 3, 9, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,15, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,15, 7},
					{15, 1, 2, 3, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,13, 0,16,11, 8, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7, 7, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7, 7, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 7, 7, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,16, 7, 7, 7, 7},
					{15, 1, 2, 3,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,16, 8, 7, 7, 7, 7}
				};

			m_CacheId = -1;
		}

		//-------------------------------------------------------------------------//
		public override void DrawScene(PrimitiveRenderer pr)
		{
			if (m_CacheId == -1)
				m_CacheId = JPGL.OpenGL.GraphicsSystem.CreateNewCache();
			else
			{
				JPGL.OpenGL.GraphicsSystem.DisplayCache(m_CacheId);
				return;
			}

			// Load the transformation
			pr.LoadTransform(m_Transform);

			// Yes, this is very ugly, but it is only going to be done on
			// the first draw so we can create a cache...

			// Do a pass of the tiletable, so we can collect the count for 
			// each texture;
			int[] counts = new int[m_TextureTable.Length];
			for (int r=0; r<m_Rows; r++)
			{
				for (int c=0; c<m_Cols; c++) 
					counts[(m_TileTable[c,r])]++;
			}
			
			// Loop through the texture table, and for each texture, we want
			// to build a dynamic geometry with all quads that use that tex-
			// ture, so we can draw the sets in one pass per texture.
			GroundPrimitive grndprim = new GroundPrimitive();
			grndprim.Geometry        = new GroundGeometry();
			for (int i=0; i<m_TextureTable.Length; i++)
			{
				if (counts[i] == 0) continue;

				grndprim.Geometry.Vertices  = new float[counts[i]*12];
				grndprim.Geometry.Texels    = new float[counts[i]*8];
				grndprim.Geometry.Normals   = new float[counts[i]*12];
				grndprim.Geometry.VertexCount = counts[i]*4;

				int count = 0;
				for (int r=0; r<m_Rows; r++)
				{
					for (int c=0; c<m_Cols; c++)
					{
						if (m_TileTable[c,r] == i)
						{
							float xoffset = r - (m_Rows*0.5f);
							float zoffset = c - (m_Cols*0.5f);

							grndprim.Geometry.Vertices[(count*12)+0] = -0.5f + xoffset;
							grndprim.Geometry.Vertices[(count*12)+1] = 0;
							grndprim.Geometry.Vertices[(count*12)+2] = 0.5f + zoffset;
							grndprim.Geometry.Texels[(count)*8+0] = 0;
							grndprim.Geometry.Texels[(count)*8+1] = 0;

							grndprim.Geometry.Vertices[(count*12)+3] = 0.5f + xoffset;
							grndprim.Geometry.Vertices[(count*12)+4] = 0;
							grndprim.Geometry.Vertices[(count*12)+5] = 0.5f + zoffset;
							grndprim.Geometry.Texels[(count)*8+2] = 1;
							grndprim.Geometry.Texels[(count)*8+3] = 0;

							grndprim.Geometry.Vertices[(count*12)+6] = 0.5f + xoffset;
							grndprim.Geometry.Vertices[(count*12)+7] = 0;
							grndprim.Geometry.Vertices[(count*12)+8] = -0.5f + zoffset;
							grndprim.Geometry.Texels[(count)*8+4] = 1;
							grndprim.Geometry.Texels[(count)*8+5] = 1;

							grndprim.Geometry.Vertices[(count*12)+ 9] = -0.5f + xoffset;
							grndprim.Geometry.Vertices[(count*12)+10] = 0;
							grndprim.Geometry.Vertices[(count*12)+11] = -0.5f + zoffset;
							grndprim.Geometry.Texels[(count)*8+6] = 0;
							grndprim.Geometry.Texels[(count)*8+7] = 1;

							count++;
						}						
					}
				}

				// Normals are always <0,1,0>
				for (int n=0; n<counts[i]*4; n++)
				{
					grndprim.Geometry.Normals[n*3+0] = 0;
					grndprim.Geometry.Normals[n*3+1] = 1;
					grndprim.Geometry.Normals[n*3+2] = 0;
				}
				
				// Draw the primitive
				grndprim.Texture  = m_TextureTable[i];
				pr.DrawPrimitive(grndprim);	
			}

			// Unload the transformation
			pr.UnloadTransform();

			JPGL.OpenGL.GraphicsSystem.EndNewCache();
		}


		#region Dispose

		/// <summary>
		/// Dispose
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected void Dispose( bool disposing )
		{
			if( disposing  && !_disposed )
			{
				m_OdePlane.Dispose();
			}
			_disposed = true;
		}

		private bool _disposed;

		#endregion
	}
}
