using System;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace DrivingGame
{
	/// <summary>
	/// Summary description for Skybox.
	/// </summary>
	public class Skybox : SceneNode
	{
		private SceneNode m_Top;
		private SceneNode m_Bottom;
		private SceneNode m_Left;
		private SceneNode m_Right;
		private SceneNode m_Back;
		private SceneNode m_Front;

		//-------------------------------------------------------------------------//
		public Skybox() : base(null, new Transform())
		{
			m_Top = new SceneNode(new Rectangle(), new Transform());
			m_Top.P.MoveTo(new Vector3());
			m_Top.P.SizeTo(new Vector3(1, 1, 0));
			m_Top.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxtop.jpg");
			m_Top.P.Texture.SetTextureTiled();
			m_Top.P.Lighted = false;
			m_Top.T.RXBy = -90;

			m_Bottom = new SceneNode(new Rectangle(), new Transform());
			m_Bottom.P.MoveTo(new Vector3());
			m_Bottom.P.SizeTo(new Vector3(1, 1, 0));
			m_Bottom.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxbottom.jpg");
			m_Bottom.P.Lighted = false;
			m_Bottom.P.Texture.SetTextureTiled();
			m_Bottom.T.RXBy = 90;

			m_Left = new SceneNode(new Rectangle(), new Transform());
			m_Left.P.MoveTo(new Vector3());
			m_Left.P.SizeTo(new Vector3(1, 1, 0));
			m_Left.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxleft.jpg");
			m_Left.P.Lighted = false;
			m_Left.P.Texture.SetTextureTiled();
			m_Left.T.RYBy = -90;

			m_Right = new SceneNode(new Rectangle(), new Transform());
			m_Right.P.MoveTo(new Vector3());
			m_Right.P.SizeTo(new Vector3(1, 1, 0));
			m_Right.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxright.jpg");
			m_Right.P.Lighted = false;
			m_Right.P.Texture.SetTextureTiled();
			m_Right.T.RYBy = 90;

			m_Front = new SceneNode(new Rectangle(), new Transform());
			m_Front.P.MoveTo(new Vector3());
			m_Front.P.SizeTo(new Vector3(1, 1, 0));
			m_Front.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxfront.jpg");
			m_Front.P.Lighted = false;
			m_Front.P.Texture.SetTextureTiled();

			m_Back = new SceneNode(new Rectangle(), new Transform());
			m_Back.P.MoveTo(new Vector3());
			m_Back.P.SizeTo(new Vector3(1, 1, 0));
			m_Back.P.Texture = JPGL.OpenGL.GraphicsSystem.LoadTexture("skyboxback.jpg");
			m_Back.P.Lighted = false;
			m_Back.P.Texture.SetTextureTiled();
			m_Back.T.RYBy = 180;


			float dist = 50;

			m_Top.T.T    = new Vector3(0, 1, 0) * dist;
			m_Bottom.T.T = new Vector3(0, -1, 0) * dist;
			m_Left.T.T   = new Vector3(-1, 0, 0) * dist;
			m_Right.T.T  = new Vector3(1, 0, 0) * dist;
			m_Front.T.T  = new Vector3(0, 0, 1) * dist;
			m_Back.T.T   = new Vector3(0, 0, -1) * dist;

			float tan45  = (float)System.Math.Tan(System.Math.PI / 4.0);
			m_Top.T.S    = new Vector3(1, 1, 1) * (tan45 * dist);
			m_Bottom.T.S = new Vector3(1, 1, 1) * (tan45 * dist);
			m_Left.T.S   = new Vector3(1, 1, 1) * (tan45 * dist);
			m_Right.T.S  = new Vector3(1, 1, 1) * (tan45 * dist);
			m_Front.T.S  = new Vector3(1, 1, 1) * (tan45 * dist);
			m_Back.T.S   = new Vector3(1, 1, 1) * (tan45 * dist);
		}

		//-------------------------------------------------------------------------//
		// size is distance of each skybox wall to center
		public void DrawSkybox(Camera c, PrimitiveRenderer r)
		{
			T.T = c.EyePosition;
			//T.RQ = new Quaternion(

			r.LoadTransform(T);

			JPGL.OpenGL.GraphicsSystem.StartBackgroundDraw();

			m_Top.DrawScene(r);
			m_Bottom.DrawScene(r);
			m_Left.DrawScene(r);
			m_Right.DrawScene(r);
			m_Front.DrawScene(r);
			m_Back.DrawScene(r);

			JPGL.OpenGL.GraphicsSystem.EndBackgroundDraw();

			r.UnloadTransform();
		}
	}
}
