using System;
using System.Collections;
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;


namespace DrivingGame
{
	/// <summary>
	/// Summary description for Windmill.
	/// </summary>
	public class Windmill : SceneNode
	{
		private SceneNode m_Wheel;

		public Windmill(Ode.Space sp) : base(null, new Transform())
		{
			T.T.X = 48; T.T.Z = 22;
			T.S.X = T.S.Y = T.S.Z = 10;

			ArrayList ids; Geometry.LoadMultiOBJ("windmill.obj", out ids);

			Texture t1 = JPGL.OpenGL.GraphicsSystem.LoadTexture("Buildings\\mill_1.jpg");
			Texture t2 = JPGL.OpenGL.GraphicsSystem.LoadTexture("Buildings\\mill_2.jpg");
			Texture t3 = JPGL.OpenGL.GraphicsSystem.LoadTexture("Buildings\\sails.jpg");

			// Base
			SceneNode s = new SceneNode(new LoadedPrimitive((int)ids[0]), new Transform());
			s.P.Texture = t2;
			AddChild(s);

			// Body
			s = new SceneNode(new LoadedPrimitive((int)ids[1]), new Transform());
			s.P.Texture = t1;
			AddChild(s);

			// Wheel
			s = new SceneNode(new LoadedPrimitive((int)ids[2]), new Transform());
			s.P.Texture = t3;
			AddChild(s);

			// Sails
			m_Wheel = new SceneNode(new LoadedPrimitive((int)ids[3]), new Transform());
			m_Wheel.T.T.Y = 0.72f;
			m_Wheel.T.T.Z = 0.26f;
			m_Wheel.P.Texture = t3;
			AddChild(m_Wheel);

			// Door
			s = new SceneNode(new LoadedPrimitive((int)ids[4]), new Transform());
			s.P.Texture = t2;
			AddChild(s);

			// Window
			s = new SceneNode(new LoadedPrimitive((int)ids[5]), new Transform());
			s.P.Texture = t2;
			AddChild(s);

			Ode.CappedCylinderGeom b = new Ode.CappedCylinderGeom(sp, 0.24f*T.S.X, 1.0f*T.S.Y);
			b.Position = new Ode.Vector3(T.T.X, 0.5f+T.T.Y, T.T.Z);

			Quaternion q = new Quaternion(new Vector3(1, 0, 0), 90);
			Ode.Quaternion qode = new Ode.Quaternion();
			qode.W = q.W; qode.X = q.X; qode.Y = q.Y; qode.Z = q.Z;

			b.Quaternion = qode;
		}

		public void Animate(float timestep)
		{
			m_Wheel.T.RZBy = 10.0f * timestep;
		}
	}
}
