using System;
using System.Windows.Forms; // For KeyEventArgs
using JPGL.Common;
using JPGL.Common.Primitives;
using JPGL.Math;


namespace DrivingGame
{
	/// <summary>
	/// Summary description for Viewport_Main.
	/// </summary>
	public class Viewport_Main : JPGL.OpenGL.OpenGLViewport
	{
		//-----------------------------------//
		private WorldModel m_WorldModel;
		//-----------------------------------//

		//-----------------------------------//
		private Vector3 m_LastMPos;          // Last mouse position
		//-----------------------------------//

		#region Camera Manipulation
		//-----------------------------------//
		public enum CameraViewType 
		{ 
			BEHIND, FARBEHIND, FRONT, OVERHEAD, FREE, DRIVER, CINEMATIC
		};
		//-----------------------------------//
		private CameraViewType m_CameraView;
		public CameraViewType CameraView
		{
			get { return m_CameraView;  }
			set 
			{ 
				m_CameraView = value;
				if (value == CameraViewType.FREE) 
					AttachToCamera(new Camera());

				if      (m_CameraView == CameraViewType.BEHIND)
					StatusText = "Chase Camera Selected";
				else if (m_CameraView == CameraViewType.FARBEHIND)
					StatusText = "Far Chase Camera Selected";
				else if (m_CameraView == CameraViewType.FRONT)
					StatusText = "Forward Chase Camera Selected";
				else if (m_CameraView == CameraViewType.OVERHEAD)
					StatusText = "Overhead Camera Selected";
				else if (m_CameraView == CameraViewType.FREE)
					StatusText = "Free Camera Selected";
				else if (m_CameraView == CameraViewType.DRIVER)
					StatusText = "Driver Camera Selected";
				else if (m_CameraView == CameraViewType.CINEMATIC)
					StatusText = "Drive-by Camera Selected";
			}
		}
		//-----------------------------------//
		#endregion

		//-----------------------------------//
		private Rectangle m_Speedometer;
		private bool      m_ShowHUD;
		private Texture   m_DialTexture;
		private Texture   m_NeedleTexture;
		//-----------------------------------//

		//-----------------------------------//
		public bool ShowHUD
		{
			get { return m_ShowHUD;  }
			set { m_ShowHUD = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		private string m_StatusText;
		private int    m_StatusTextTime;
		//-----------------------------------//

		//-----------------------------------//
		public string StatusText
		{
			set 
			{
				m_StatusText = value;
				m_StatusTextTime = 120;
			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public Viewport_Main(WorldModel w) : base()
		{
			m_WorldModel = w;
			m_LastMPos = new Vector3();
			m_CameraView = CameraViewType.FREE;

			Camera.Far = 120;

			// Setup HUD
			m_ShowHUD = true;

			m_Speedometer = new Rectangle();
			m_Speedometer.MoveTo(new Vector3(64, 64, 0));
			m_Speedometer.SizeTo(new Vector3(128, 128, 0));
			m_Speedometer.Lighted = false;
			m_Speedometer.Blended = true;

			m_DialTexture   = JPGL.OpenGL.GraphicsSystem.LoadTexture("Dials\\speedometer.png");
			m_NeedleTexture = JPGL.OpenGL.GraphicsSystem.LoadTexture("Dials\\needle.png");

			m_NeedleTexture.AddressingU = Texture.AddressingMethod.ADDRESSING_METHOD_CLAMP_TO_EDGE;
			m_NeedleTexture.AddressingV = Texture.AddressingMethod.ADDRESSING_METHOD_CLAMP_TO_EDGE;

			// Status text
			m_StatusText     = "";
			m_StatusTextTime = 0;
		}

		//-------------------------------------------------------------------------//
		public void Reset()
		{
			if (m_CameraView == CameraViewType.FREE) 
				AttachToCamera(new Camera());
		}

		//-------------------------------------------------------------------------//
		protected override void Draw()
		{
			#region Update Camera
			if (m_CameraView == CameraViewType.BEHIND ||
				m_CameraView == CameraViewType.FARBEHIND ||
				m_CameraView == CameraViewType.FRONT)
			{
				// Look at pos is car
				Camera.LookAtPosition.X = m_WorldModel.Car1.T.T.X;
				Camera.LookAtPosition.Y = m_WorldModel.Car1.T.T.Y;
				Camera.LookAtPosition.Z = m_WorldModel.Car1.T.T.Z;

				float x=0,y=0,z=0;
				switch (m_CameraView)
				{
					case (CameraViewType.BEHIND):
						y = 1; z = -2;
						break;
					case (CameraViewType.FRONT):
						y = 1; z = 2;
						break;
					case (CameraViewType.FARBEHIND):
						y = 1; z = -5;
						break;
				}

				// Get Car direction
				Vector3 lookdir = new Matrix(
					m_WorldModel.Car1.CarBody.Rotation.m11, m_WorldModel.Car1.CarBody.Rotation.m21, m_WorldModel.Car1.CarBody.Rotation.m31, 0,
					m_WorldModel.Car1.CarBody.Rotation.m12, m_WorldModel.Car1.CarBody.Rotation.m22, m_WorldModel.Car1.CarBody.Rotation.m32, 0,
					m_WorldModel.Car1.CarBody.Rotation.m13, m_WorldModel.Car1.CarBody.Rotation.m23, m_WorldModel.Car1.CarBody.Rotation.m33, 0,
					0, 0, 0, 1).Transform(new Vector3(x, y, z));

				Camera.EyePosition.X = Camera.LookAtPosition.X + 5.0f * lookdir.X;
				Camera.EyePosition.Y = System.Math.Abs(Camera.LookAtPosition.Y + 5.0f * lookdir.Y);
				Camera.EyePosition.Z = Camera.LookAtPosition.Z + 5.0f * lookdir.Z;

				Camera.EyeUpDirection.X = 0;
				Camera.EyeUpDirection.Y = 1;
				Camera.EyeUpDirection.Z = 0;
			
				Camera.FOV = Camera.FOV; // Forces Camera.OnUpdate()
			}
			else if (m_CameraView == CameraViewType.OVERHEAD)
			{
				// Look at pos is car
				Camera.LookAtPosition.X = m_WorldModel.Car1.T.T.X;
				Camera.LookAtPosition.Y = m_WorldModel.Car1.T.T.Y;
				Camera.LookAtPosition.Z = m_WorldModel.Car1.T.T.Z;

				Camera.EyePosition.X = m_WorldModel.Car1.T.T.X;
				Camera.EyePosition.Y = m_WorldModel.Car1.T.T.Y + 35.0f;
				Camera.EyePosition.Z = m_WorldModel.Car1.T.T.Z;
			
				Camera.EyeUpDirection.X = m_WorldModel.Car1.CarBody.Rotation.m31;
				Camera.EyeUpDirection.Y = m_WorldModel.Car1.CarBody.Rotation.m32;
				Camera.EyeUpDirection.Z = m_WorldModel.Car1.CarBody.Rotation.m33;

				Camera.FOV = Camera.FOV; // Forces Camera.OnUpdate()
			}
			else if (m_CameraView == CameraViewType.CINEMATIC)
			{
				Camera.LookAtPosition.X = m_WorldModel.Car1.T.T.X;
				Camera.LookAtPosition.Y = m_WorldModel.Car1.T.T.Y;
				Camera.LookAtPosition.Z = m_WorldModel.Car1.T.T.Z;

				Camera.EyePosition.X = 0;
				Camera.EyePosition.Y = 20;
				Camera.EyePosition.Z = 0;

				Camera.EyeUpDirection = (Camera.LookAtPosition - Camera.EyePosition).CrossProduct(new Vector3(0, 1, 0)).CrossProduct(Camera.LookAtPosition - Camera.EyePosition);

			}
			else if (m_CameraView == CameraViewType.DRIVER)
			{
				Vector3 bodyoffset = new Matrix(
					m_WorldModel.Car1.CarBody.Rotation.m11, m_WorldModel.Car1.CarBody.Rotation.m21, m_WorldModel.Car1.CarBody.Rotation.m31, 0,
					m_WorldModel.Car1.CarBody.Rotation.m12, m_WorldModel.Car1.CarBody.Rotation.m22, m_WorldModel.Car1.CarBody.Rotation.m32, 0,
					m_WorldModel.Car1.CarBody.Rotation.m13, m_WorldModel.Car1.CarBody.Rotation.m23, m_WorldModel.Car1.CarBody.Rotation.m33, 0,
					0, 0, 0, 1).Transform(new Vector3(0, 0.5f*m_WorldModel.Car1.BodySize.Y, 0));
				Camera.EyePosition.X = m_WorldModel.Car1.CarBody.Position.X + bodyoffset.X;
				Camera.EyePosition.Y = m_WorldModel.Car1.CarBody.Position.Y + bodyoffset.Y;
				Camera.EyePosition.Z = m_WorldModel.Car1.CarBody.Position.Z + bodyoffset.Z;

				Camera.EyeUpDirection.X = m_WorldModel.Car1.CarBody.Rotation.m21;
				Camera.EyeUpDirection.Y = m_WorldModel.Car1.CarBody.Rotation.m22;
				Camera.EyeUpDirection.Z = m_WorldModel.Car1.CarBody.Rotation.m23;

				Camera.LookAtPosition.X = Camera.EyePosition.X + m_WorldModel.Car1.CarBody.Rotation.m31;
				Camera.LookAtPosition.Y = Camera.EyePosition.Y + m_WorldModel.Car1.CarBody.Rotation.m32;
				Camera.LookAtPosition.Z = Camera.EyePosition.Z + m_WorldModel.Car1.CarBody.Rotation.m33;

				Camera.FOV = Camera.FOV; // Forces Camera.OnUpdate()
			}
			#endregion

			// Draw the skybox
			m_WorldModel.Skybox.DrawSkybox(this.Camera, PrimRenderer);

			// Draw the world
			m_WorldModel.DrawWorld(PrimRenderer);

			EnterOrthoMode();
			if (WorldModel.EchoString != null)
			{
				StatusText = WorldModel.EchoString;
				WorldModel.EchoString = null;
			}
			if (m_StatusTextTime > 0)
			{
				// Draw Status Text
				JPGL.OpenGL.GraphicsSystem.Print(
					m_Width-(20*m_StatusText.Length)-12, m_Height-32, // Position
					1, 1,                    // Scale
					m_StatusText, 0, // String and charset
					1, 1, 1);                // Color

				m_StatusTextTime--;
			}

			// Draw Speedometer
			if (m_ShowHUD)
			{
				m_Speedometer.Texture = m_DialTexture;
				PrimRenderer.DrawPrimitive(m_Speedometer);

				m_NeedleTexture.Rotation = m_WorldModel.Car1.CurrentSpeed * 3.6f;
				m_Speedometer.Texture = m_NeedleTexture;
				PrimRenderer.DrawPrimitive(m_Speedometer);
			}
			ExitOrthoMode();
		}

		//-------------------------------------------------------------------------//
		public override void OnMouseDown(int mx, int my, uint nDown)
		{
			if (m_CameraView != CameraViewType.FREE) return; 

			// Translate the mouse coordinates to device coords
			int dx, dy; HardwareToDevice(mx, my, out dx, out dy);

			// If a button is down, record position
			if ((nDown & JPGL.SWF.UIWindow.MB_LEFT) != 0 ||
				(nDown & JPGL.SWF.UIWindow.MB_RIGHT) != 0 ||
				(nDown & JPGL.SWF.UIWindow.MB_MIDDLE) != 0 )
			{
				m_LastMPos = new Vector3(dx, dy, 0);
			}
		}

		//-------------------------------------------------------------------------//
		public override void OnMouseMove(int hx, int hy, uint nDown)
		{
			if (m_CameraView != CameraViewType.FREE) return; 

			// Translate the mouse coordinates to device coords
			int dx, dy; HardwareToDevice(hx, hy, out dx, out dy);

			Vector3 delta = new Vector3(dx, dy, 0) - m_LastMPos;

			// If LMB is down, rotate camera
			if ((nDown & JPGL.SWF.UIWindow.MB_LEFT) != 0)
			{
				Camera.RotateAroundPoint(Camera.LookAtPosition, delta.X, new Vector3(0, 1, 0));
				Camera.RotateAroundPoint(Camera.LookAtPosition, delta.Y, new Vector3(1, 0, 0));		
			}

			// If RMB is down, zoom
			if ((nDown & JPGL.SWF.UIWindow.MB_RIGHT) != 0)
				Camera.Zoom(delta.X * 0.5f);

			// If MMB is down, strafe
			if ((nDown & JPGL.SWF.UIWindow.MB_MIDDLE) != 0)
				Camera.Strafe(new Vector3(delta.X, delta.Y, delta.Z) * 0.25f);

			m_LastMPos = new Vector3(dx, dy, 0);
		}

		//-------------------------------------------------------------------------//
		public void KeyDown(KeyEventArgs e)
		{
			if      (e.KeyCode == Keys.A) m_WorldModel.CarSteering = WorldModel.Steering.LEFT;
			else if (e.KeyCode == Keys.D) m_WorldModel.CarSteering = WorldModel.Steering.RIGHT;
			else if (e.KeyCode == Keys.W) m_WorldModel.CarSpeed = WorldModel.Speed.FORWARD;
			else if (e.KeyCode == Keys.S) m_WorldModel.CarSpeed = WorldModel.Speed.REVERSE;
			else if (e.KeyCode == Keys.Space) m_WorldModel.Handbrake = true;
			else if (e.KeyCode == Keys.F) m_WorldModel.Car1.Flip();
			else if (e.KeyCode == Keys.C)
			{
				int newview = (int)m_CameraView + 1;
				if (newview > (int)CameraViewType.CINEMATIC) newview = (int)CameraViewType.BEHIND;
				CameraView = (CameraViewType)newview;
			}
		}

		//-------------------------------------------------------------------------//
		public void KeyUp(KeyEventArgs e)
		{
			if      (e.KeyCode == Keys.A) m_WorldModel.CarSteering = WorldModel.Steering.NONE;
			else if (e.KeyCode == Keys.D) m_WorldModel.CarSteering = WorldModel.Steering.NONE;
			else if (e.KeyCode == Keys.W) m_WorldModel.CarSpeed = WorldModel.Speed.NONE;
			else if (e.KeyCode == Keys.S) m_WorldModel.CarSpeed = WorldModel.Speed.NONE;
			else if (e.KeyCode == Keys.Space) m_WorldModel.Handbrake = false;
		}
	}
}