using System;
using JPGL.Common;
using JPGL.Common.Viewports;
using JPGL.Math;


namespace DrivingGame
{
	/// <summary>
	/// Summary description for Viewport_Overview.
	/// </summary>
	public class Viewport_Overview : JPGL.OpenGL.OpenGLViewport /**JPGL.DirectX.DirectXViewport*/
	{
		//-----------------------------------//
		private WorldModel m_WorldModel;
		//-----------------------------------//

		//-----------------------------------//
		private Vector3 m_LastMPos;          // Last mouse position
		//-----------------------------------//

		//-----------------------------------//
		private Viewport m_OtherView;        // Viewport whos camera we will show
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public Viewport_Overview(WorldModel w, Viewport other) : base()
		{
			m_WorldModel = w;
			m_OtherView  = other;

			m_LastMPos = new Vector3();
		}

		//-------------------------------------------------------------------------//
		public void Reset()
		{
			AttachToCamera(new Camera());
		}

		//-------------------------------------------------------------------------//
		protected override void Draw()
		{
			m_WorldModel.Skybox.DrawSkybox(this.Camera, PrimRenderer);

			// Draw the other guy's camera
			m_OtherView.Camera.DrawCamera(PrimRenderer);

			// Draw the world
			m_WorldModel.DrawWorld(PrimRenderer);
		}

		//-------------------------------------------------------------------------//
		public override void OnMouseDown(int mx, int my, uint nDown)
		{
			// Translate the mouse coordinates to device coords
			int dx, dy; HardwareToDevice(mx, my, out dx, out dy);

			// If a button is down, record position
			if ((nDown & JPGL.SWF.UIWindow.MB_LEFT) != 0 ||
				(nDown & JPGL.SWF.UIWindow.MB_RIGHT) != 0 ||
				(nDown & JPGL.SWF.UIWindow.MB_MIDDLE) != 0 )
			{
				m_LastMPos = new Vector3(dx, dy, 0);
			}
		}

		//-------------------------------------------------------------------------//
		public override void OnMouseMove(int hx, int hy, uint nDown)
		{
			// Translate the mouse coordinates to device coords
			int dx, dy; HardwareToDevice(hx, hy, out dx, out dy);

			Vector3 delta = new Vector3(dx, dy, 0) - m_LastMPos;

			// If LMB is down, rotate camera
			if ((nDown & JPGL.SWF.UIWindow.MB_LEFT) != 0)
			{
				Camera.RotateAroundPoint(Camera.LookAtPosition, delta.X, new Vector3(0, 1, 0));
				Camera.RotateAroundPoint(Camera.LookAtPosition, delta.Y, new Vector3(1, 0, 0));		
			}

			// If RMB is down, zoom
			if ((nDown & JPGL.SWF.UIWindow.MB_RIGHT) != 0)
				Camera.Zoom(delta.X * 0.5f);

			// If MMB is down, strafe
			if ((nDown & JPGL.SWF.UIWindow.MB_MIDDLE) != 0)
				Camera.Strafe(new Vector3(delta.X, delta.Y, delta.Z) * 0.25f);

			m_LastMPos = new Vector3(dx, dy, 0);
		}
	}
}