using System;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace JPGL.Common.Colliders
{
	/// <summary>
	/// 
	/// </summary>
	public abstract class Collider
	{
		//-----------------------------------//
		protected Vector3 m_Position;        // Position of this collider
		public Vector3 Position
		{
			get { return m_Position;  }
			set { m_Position = value; }
		}
		protected Vector3 m_Velocity;        // Velocity of this collider
		public Vector3 Velocity
		{
			get { return m_Velocity;  }
			set { m_Velocity = value; }
		}
		//-----------------------------------//

		//-----------------------------------// Colliders must have some
		public abstract float Size           // notion of their size - 
		{                                    // note that this size has
			get;                             // no units, so is only
			set;                             // usefull for relative
		}                                    // sizing operations...
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public Collider(Vector3 position)
		{
			m_Position = new Vector3(position);
			m_Velocity = new Vector3();
		}

		//-------------------------------------------------------------------------//
		public void DrawCollider(PrimitiveRenderer r)
		{
			Draw(r);
		}

		//-------------------------------------------------------------------------//
		// 
		public bool CollidesWith(Collider other)
		{
			// Get position in other collider closest to this collider's adjusted position
			Vector3 adjPos = other.GetAdjustedPosition(Position);

			// Get this colliders closest position to the other collider adjusted position
			Vector3 ourPos = GetAdjustedPosition(adjPos);

			// Check if the other's adjusted position is in this collider
			if (IsInCollider(adjPos)) return true;

			// Check if this adjusted position is in the other's collider
			if (other.IsInCollider(ourPos)) return true;

			return false;
		}

		//-------------------------------------------------------------------------//
		public bool CollidesWithOneSided(Collider other)
		{
			// Get position in other collider closest to this collider's adjusted position
			Vector3 adjPos = other.GetAdjustedPosition(Position);

			// Check if the other's adjusted position is in this collider
			if (IsInCollider(adjPos)) return true;

			return false;
		}

		//-------------------------------------------------------------------------//
		// Collide this collider with the given other collider. Both this and the
		// other collider will have the resulting velocities and positions of the
		// collision stored in themselves.
		public void DoCollision(Collider other)
		{
			Collider newThis = CollideWith(other);
			Collider newOther = other.CollideWith(this);

			this.Position = newThis.Position;
			this.Velocity = newThis.Velocity;

			other.Velocity = newOther.Velocity;
			other.Position = newOther.Position;
		}


		//-------------------------------------------------------------------------//
		// Return the position on this collider closest to the given point. Note that
		// if a position inside this collider is given, that position is the closest
		// point on this collider! This is important.
		public abstract Vector3 GetAdjustedPosition(Vector3 v);

		//-------------------------------------------------------------------------//
		// Return true if the given point is in this collider.
		public abstract bool IsInCollider(Vector3 v);

		//-------------------------------------------------------------------------//
		// Given another collider, collider that collider with this collider. Only
		// the other collider will be updated, not this collider.
		public abstract Collider CollideWith(Collider other);

		//-------------------------------------------------------------------------//
		// Draw a representation of the collider.
		protected abstract void Draw(PrimitiveRenderer r);
	}
}
