using System;
using JPGL.Common.Primitives;
using JPGL.Math;


namespace JPGL.Common.Colliders
{
	/// <summary>
	/// Summary description for ColliderCircle.
	/// </summary>
	public class ColliderCircle : Collider
	{
		//-----------------------------------//
		private float m_Radius;              //
		public float Radius
		{
			get { return m_Radius; }
		}
		public override float Size
		{
			get { return m_Radius; }
			set { m_Radius = value; }
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public ColliderCircle(Vector3 p, float r) : base(p)
		{
			m_Radius = r;
		}

		//-------------------------------------------------------------------------//
		// Return the position on this collider closest to the given point.
		public override Vector3 GetAdjustedPosition(Vector3 v)
		{
			// Is the given point in this collider?
			if (IsInCollider(v)) return v;

			// Get vector pointing from Position to v
			Vector3 direction = (v - Position).Normalize();

			// Return the direction scaled by the radius and added to position
			return ((direction * m_Radius) + Position);
		}

		//-------------------------------------------------------------------------//
		// Return true if the given point is in this collider.
		public override bool IsInCollider(Vector3 v)
		{
			// Find the distance from the point to our position
			Vector3 delta = Position - v;

			// Is the distance less than radius?
			return (delta.Length() <= m_Radius);
		}

		//-------------------------------------------------------------------------//
		public override Collider CollideWith(Collider other)
		{
			Collider result = new ColliderCircle(Position, m_Radius);

			// Get adjusted positions
			Vector3 adjPos = other.GetAdjustedPosition(Position);
			Vector3 myAPos = GetAdjustedPosition(adjPos);

			// Get the direction of the axis between adjusted positions
			Vector3 axis = (adjPos - Position).Normalize();

			// If not moving, kickstart us along the neg of the axis' direction
			if (Velocity.X == 0.0f && Velocity.Y == 0.0f && Velocity.Z == 0.0f)
			{
				result.Velocity = (axis * -1.0f) * other.Velocity.Length();
			}
			else
			{
				Vector3 i = Velocity.Normalize();
				result.Velocity = ((axis * (((i * -1.0f) * axis) * 2.0f)) + i) * Velocity.Length();
			}

			result.Position = adjPos + ((Position - adjPos).Normalize() * m_Radius);
			return result;
		}

		//-------------------------------------------------------------------------//
		// Draw a representation of the collider.
		protected override void Draw(PrimitiveRenderer r)
		{
			DrawablePrimitive c = new Circle();
			c.MoveTo(Position);
			c.SizeTo(new Vector3(Position.X + m_Radius, Position.Y, Position.Z));
			c.Filled = false;
			c.Shaded = false;
			c.Color = System.Drawing.Color.Black;
			r.DrawPrimitive(c);
		}
	}
}
