using System;
using JPGL.Math;

namespace JPGL.Common.Colliders
{
	/// <summary>
	/// Summary description for ColliderClipPlane.
	/// </summary>
	public class ColliderClipPlane : Collider
	{
		//-----------------------------------//
		private Vector3 m_Normal;            // The plane's normal vector
		//-----------------------------------//
		public Vector3 Normal
		{
			get { return m_Normal; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public override float Size
		{
			get { return 1; }
			set {  }
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public ColliderClipPlane(Vector3 p, Vector3 n) : base(p)
		{
			m_Normal = n;
		}

		//-------------------------------------------------------------------------//
		public override Vector3 GetAdjustedPosition(Vector3 v)
		{
			Vector3 incl = Position - v;

			// Project incl onto the normal
			Vector3 projnI = m_Normal * (incl * m_Normal);

			return (v + projnI);
		}

		//-------------------------------------------------------------------------//
		public override bool IsInCollider(Vector3 v)
		{
			// Plane Equation => Ax + By + Cz + D = 0

			// Calculate D
			float D = -1.0f * ((m_Normal.X * m_Position.X) + (m_Normal.Y * m_Position.Y) + (m_Normal.Z * m_Position.Z));

			// Calculate distance point to plane
			float distance = ((m_Normal.X * v.X)  +		// Ax +
							  (m_Normal.Y * v.Y)  +		// Bx +
							  (m_Normal.Z * v.Z)) + D;	// Cz + D

			// dist < 0, point on other side of plane
			// dist = 0, point is on plane
			// dist > 0, point on okay side of plane
			return (distance <= 0);
		}

		//-------------------------------------------------------------------------//
		public override Collider CollideWith(Collider other)
		{


			return this;
		}

		//-------------------------------------------------------------------------//
		protected override void Draw(JPGL.Common.Primitives.PrimitiveRenderer r)
		{
			
		}
	}
}
