using System;
using System.Collections;
using JPGL.Common.Primitives;
using JPGL.Math;

namespace JPGL.Common.Colliders
{
	/// <summary>
	/// ColliderList is a list of colliders.
	/// </summary>
	public class ColliderList : IEnumerable
	{
		//-----------------------------------//
        private ArrayList m_Colliders;       //
		//-----------------------------------//
		

		//-------------------------------------------------------------------------//
		public ColliderList()
		{
			m_Colliders = new ArrayList();
		}

		//-------------------------------------------------------------------------//
		public ColliderList(Collider c)
		{
			m_Colliders = new ArrayList();
			m_Colliders.Add(c);
		}

		//-------------------------------------------------------------------------//
		public void AddCollider(Collider c)
		{
			m_Colliders.Add(c);
		}

		//-------------------------------------------------------------------------//
		public void Draw(PrimitiveRenderer r)
		{
			foreach (Collider c in m_Colliders)
			{
				c.DrawCollider(r);
			}
		}

		//-------------------------------------------------------------------------//
		public void Clear()
		{
			m_Colliders.Clear();
		}


		//-------------------------------------------------------------------------//
		// Enumeration functions (so we can use foreach ... in ColliderList)
		public IEnumerator GetEnumerator()
		{
			return new ColliderListEnumerator(this);
		} 

		/// <summary>
		/// Private Enumerator so we can use foreach on our lists.
		/// </summary>
		private class ColliderListEnumerator : IEnumerator
		{
			//-----------------------------------//
			private ColliderList m_List;         // Reference to list
			private int m_Position;              // Position used for enumeration
			//-----------------------------------//

			public ColliderListEnumerator(ColliderList list)
			{
				m_List     = list;
				m_Position = -1;
			}

			//-------------------------------------------------------------------------//
			public bool MoveNext()
			{
				m_Position++;
				if (m_Position >= m_List.m_Colliders.Count)
				{
					return false;
				}

				return true;
			} 

			//-------------------------------------------------------------------------//
			public void Reset()
			{
				m_Position = -1;
			} 

			//-------------------------------------------------------------------------//
			public object Current
			{
				get
				{
					return (m_List.m_Colliders[m_Position]);
				}
			}
		}
	}
}
