using System;
using JPGL.Math;
using JPGL.Common.Primitives;


namespace JPGL.Common.Lighting
{
	/// <summary>
	/// Summary description for SpotLight.
	/// </summary>
	public class SpotLight : Light
	{
		//-----------------------------------//
		public override int LightType
		{
			get
			{
				return Light.LIGHT_SPOT;
			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public override void DrawLight(PrimitiveRenderer r)
		{
			// Cone
			SceneNode c = new SceneNode(new LoadedPrimitive(JPGL.Common.SceneNodes.Arrow.CONEGEOMETRY), new Transform());
			c.P.Lighted = false;
			c.P.Filled = false;
			c.P.Color  = (m_On ? m_DiffuseColor : System.Drawing.Color.Black);
			c.T.S = new Vector3(0.25f, 0.25f, 0.25f);
			c.T.T = m_Position;
			c.T.RQ = new Quaternion(
				new Vector3(0, 0, 1).CrossProduct(m_Direction).Normalize(), 
				(float)System.Math.Acos(new Vector3(0, 0, 1) * m_Direction)  * (180.0f/(float)System.Math.PI));

			c.DrawScene(r);

			// Line
			SceneNode l = new SceneNode(new Line(), new Transform());
			l.P.Lighted = false;
			l.P.Color   = (m_On ? m_DiffuseColor : System.Drawing.Color.Black);
			l.P.Shaded  = false;
			l.P.MoveTo(new Vector3(-0.5f, 0, 0));
			l.P.SizeTo(new Vector3(0, 0, 0));
			l.T.T = m_Position;
			l.T.RQ = new Quaternion(
				new Vector3(-1, 0, 0).CrossProduct(m_Direction).Normalize(), 
				(float)System.Math.Acos(new Vector3(-1, 0, 0) * m_Direction)  * (180.0f/(float)System.Math.PI));

			l.DrawScene(r);
		}
	}
}
