using System;
using System.Drawing;
using System.IO;


namespace JPGL.Common
{
	/// <summary>
	/// Summary description for Material.
	/// </summary>
	public class Material
	{
		#region Private Members and Public Accessors
		//-----------------------------------//
		protected Color m_Ambient;
		public Color Ambient
		{
			get { return m_Ambient;  }
			set { m_Ambient = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		protected Color m_Specular;
		public Color Specular
		{
			get { return m_Specular;  }
			set { m_Specular = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		protected Color m_Diffuse;
		public Color Diffuse
		{
			get { return m_Diffuse;  }
			set { m_Diffuse = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		protected Color m_Emmisive;
		public Color Emmisive
		{
			get { return m_Emmisive;  }
			set { m_Emmisive = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		private float m_Shininess;
		public float Shininess
		{
			get { return m_Shininess;  }
			set { m_Shininess = value; }
		}
		//-----------------------------------//
		#endregion


		//-------------------------------------------------------------------------//
		public Material()
		{
			m_Ambient       = Color.FromArgb(255, 51, 51, 51);
			m_Specular      = Color.FromArgb(255, 0, 0, 0);
			m_Diffuse       = Color.FromArgb(255, 204, 204, 204);
			m_Emmisive      = Color.FromArgb(255, 0, 0, 0);
			m_Shininess     = 0;
		}

		//-------------------------------------------------------------------------//
		public Material(Material old)
		{
			m_Ambient       = old.Ambient;
			m_Specular      = old.Specular;
			m_Diffuse       = old.Diffuse;
			m_Shininess     = old.Shininess;
			m_Emmisive      = old.Emmisive;
		}

		#region Static MTL File Loader Functions
		//-------------------------------------------------------------------------//
		// Loads the first material from the given MTL file.
		//		newmtl DEFAULT                  - name
		//		Ns 0                            - shininess
		//		d 1.000                         - transparency (alpha)
		//		illum 2                         - illumination
		//		Kd 0.600 0.600 0.600            - diffuse
		//		Ks 1.000 1.000 1.000            - specular
		//		Ka 0.300 0.300 0.300            - ambient
		public static Material LoadMTL(string fileName)
		{
			string fileNameWPath;
			if (File.Exists(fileName))
				fileNameWPath = fileName;
			else if (System.IO.File.Exists("../../../Models/" + fileName))
				fileNameWPath = "../../../Models/" + fileName;
			else if (System.IO.File.Exists("Models/" + fileName))
				fileNameWPath = "Models/" + fileName;
			else
				return null;

			Material m = new Material();

			StreamReader file = new StreamReader(File.OpenRead(fileNameWPath));
			bool readingMat = false;
			string line;
			while ((line = file.ReadLine()) != null)
			{
				line = line.TrimStart(null); // remove leading whitespace chars
				if (line.Length == 0) continue;
				
				if (line[0] == 'n' && line[1] == 'e')
				{
					if (readingMat) { break; }
					readingMat = true;
					continue;
				}

				if (line[0] == 'K')
				{
					string[] tokens = line.Split(' ');

					if      (line[1] == 'd')
						m.Diffuse = Color.FromArgb((int)(float.Parse(tokens[1]) * 255.0f), (int)(float.Parse(tokens[2]) * 255.0f), (int)(float.Parse(tokens[3]) * 255.0f));
					else if (line[1] == 's')
						m.Specular = Color.FromArgb((int)(float.Parse(tokens[1]) * 255.0f), (int)(float.Parse(tokens[2]) * 255.0f), (int)(float.Parse(tokens[3]) * 255.0f));
					else if (line[1] == 'a')
						m.Ambient = Color.FromArgb((int)(float.Parse(tokens[1]) * 255.0f), (int)(float.Parse(tokens[2]) * 255.0f), (int)(float.Parse(tokens[3]) * 255.0f));
				}
				else if (line[0] == 'N' && line[1] == 's')
				{
					string[] tokens = line.Split(' ');
					m.Shininess = float.Parse(tokens[1]);
				}


			}

			return m;
		}
		//-------------------------------------------------------------------------//	
		#endregion
	}
}
