using System;
using JPGL.Math;

namespace JPGL.Common.Primitives
{
	/// <summary>
	/// DrawablePrimitive1Point provides point list access implementation for
	/// DrawablePrimitives that can be defined by a single point.
	/// </summary>
	public abstract class DrawablePrimitive1Point : DrawablePrimitive
	{
		//-----------------------------------// 
		public override bool Defined         // A 1Point primitive is defined
		{                                    // if 1 point is defined. 
			get
			{
				return (m_Points.Count >= 1);
			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		// New points are always inserted at position 0.
		protected sealed override void AddPoint(Vector3 p)
		{
			m_Points.Insert(0, p);

			Point1Added();
		}

		//-------------------------------------------------------------------------//
		protected sealed override void SetFirstPoint(Vector3 p)
		{
			m_Points.Insert(0, p);

			Point1Added();
		}

		//-------------------------------------------------------------------------//
		// Inheriting classes may override this function to perform some additional
		// action when Point 1 is added.
		protected virtual void Point1Added() {}
	}
}
