using System;
using JPGL.Math;

namespace JPGL.Common.Primitives
{
	/// <summary>
	/// A 2 Point DrawablePrimitive is a drawable primitive that is
	/// defined by 2 points. This primitive never has more than 2
	/// points, so we will override the AddPoint function of 
	/// Primitive so that the 3rd point added will reset the
	/// primitive - that is, it will become the new 1st point.
	/// </summary>
	public abstract class DrawablePrimitive2Point : DrawablePrimitive
	{
		//-----------------------------------// 
		public override bool Defined         // A 2 point primitive is defined if
		{                                    // 2 points are defined. 
			get
			{
				return (m_Points.Count >= 2);
			}
		}
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public DrawablePrimitive2Point() : base()
		{

		}

		//-------------------------------------------------------------------------//
		public DrawablePrimitive2Point(DrawablePrimitive2Point p) : base(p)
		{

		}

		//-------------------------------------------------------------------------//
		// Always added at position 0
		protected sealed override void SetFirstPoint(Vector3 p) 
		{
			m_Points.Insert(0, p);

			Point1Added();
		}

		//-------------------------------------------------------------------------//
		// Always added at position 1
		protected sealed override void AddPoint(Vector3 p)      
		{ 
			m_Points.Insert(1, p);
   
			Point2Added();
		}

		//-------------------------------------------------------------------------//
		// Classes implementing DrawablePrimitive2Point can override these two 
		// functions if they wish to perform some additional special action when
		// one of the 2 points is added.
		protected virtual void Point1Added() {}
		protected virtual void Point2Added() {}
	}
}
