using System;

namespace JPGL.Common.Primitives
{
	/// <summary>
	/// A Geometry type allowing for non-predefined (hence
	/// non-static and non-cached) Geometry definitions.
	/// </summary>
	public class DynamicGeometry : Geometry
	{
		private float[] m_Vertices;
		private float[] m_TexCoords;
		private float[] m_Normals;
		private int     m_DrawMode;

		#region Data Accessor Properties
		//-----------------------------------// Get the vertices of this object
		public override float[] Vertices
		{
			get { return m_Vertices;  }
			set { m_Vertices = value; }
		}
		//-----------------------------------//

		//-----------------------------------// Get the vertices of this object
		public override float[] Texels
		{
			get { return m_TexCoords;  }
			set { m_TexCoords = value; }
		}
		//-----------------------------------//

		//-----------------------------------// Get the vertices of this object
		public override float[] Normals
		{
			get { return m_Normals;  }
			set { m_Normals = value; }
		}
		//-----------------------------------//

		//-----------------------------------// Get number of vertices
		public override int VertexCount
		{
			get { return m_Vertices.Length/3; }
		}
		//-----------------------------------//

		//-----------------------------------// Get the drawmode of this object
		public override int DrawMode
		{
			get { return m_DrawMode;  }
			set { m_DrawMode = value; }
		}
		//-----------------------------------//
		#endregion
		
		//-------------------------------------------------------------------------//
		public DynamicGeometry(int drawMode) : base(-1)
		{
			m_DrawMode  = drawMode;
			m_Vertices  = null;
			m_TexCoords = null;
			m_Normals   = null;
		}
	}
}
