using System;
using System.Runtime.InteropServices;


namespace JPGL.Common.Primitives
{
	/// <summary>
	/// The format of a single vertex; more fun with C#'s 
	/// Runtime attributes, as we specify that we want this
	/// struct to be sequential - not optimized and re-
	/// arranged by the compiler.
	/// </summary>
	[StructLayout (LayoutKind.Sequential)]
	public struct VertexFormat
	{
		public float u,v;
		public float r,g,b,a;
		public float nX, nY, nZ;
		public float x, y, z;
	}

	/// <summary>
	/// Summary description for IndexedGeometry.
	/// </summary>
	public class IndexedGeometry : Geometry
	{
		//-----------------------------------//
		protected uint[]         m_Indices;
		protected VertexFormat[] m_Vertices;
		protected int            m_DrawMode;
		//-----------------------------------//

		#region Public Property Accessors
		//-----------------------------------//
		public uint[] Indices
		{
			get { return m_Indices;  }
			set { m_Indices = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public VertexFormat[] VertexData
		{
			get { return m_Vertices;  }
			set { m_Vertices = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public override int DrawMode
		{
			get { return m_DrawMode;  }
			set { m_DrawMode = value; }
		}
		//-----------------------------------//
		#endregion

		public IndexedGeometry() : base(-1)
		{
			m_Indexed  = true;
			m_Indices  = null;
			m_Vertices = null;
			m_DrawMode = PrimitiveRenderer.TRIANGLES;
		}
	}
}
