using System;
using JPGL.Common.Colliders;
using JPGL.Math;


namespace JPGL.Common.Primitives
{
	/// <summary>
	/// Draw multiple points in one pass
	/// </summary>
	public class MultiPointPrimitive : DrawablePrimitive
	{
		//-----------------------------------// 
		private DynamicGeometry m_Geometry;
		//-----------------------------------// 

		//-----------------------------------// 
		public override bool Defined         //
		{                                    //
			get
			{
				return (m_Geometry.VertexCount >= 1);
			}
		}
		//-----------------------------------//

		//-----------------------------------//
		public Vector3 this[int i]
		{
			set
			{
				if (i >= (m_Geometry.Vertices.Length / 3)) return;

				m_Geometry.Vertices[i*3+0] = value.X;
				m_Geometry.Vertices[i*3+1] = value.Y;
				m_Geometry.Vertices[i*3+2] = value.Z;

			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public MultiPointPrimitive() : base()
		{
			m_Geometry = new DynamicGeometry(PrimitiveRenderer.POINTS);
			m_Geometry.Texels   = null;
			this.Lighted = false;
			this.Shaded  = false;
		}

		//-------------------------------------------------------------------------//
		public void Prepare(int size)
		{
			m_Geometry.Vertices = new float[size*3];
			m_Geometry.Normals  = new float[size*3];
			for (int i=0; i<size; i++)
			{
				m_Geometry.Normals[i*3+0] = 0;
				m_Geometry.Normals[i*3+1] = 1;
				m_Geometry.Normals[i*3+2] = 0;
			}
		}

		//-------------------------------------------------------------------------//
		public override Geometry GetGeometry()
		{
			return m_Geometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return new Transform();
		}

		//-------------------------------------------------------------------------//
		public override ColliderList GetColliders()
		{
			return new ColliderList();
		}
	}
}
