using System;
using JPGL.Common.Colliders;
using JPGL.Math;


namespace JPGL.Common.Primitives
{
	/// <summary>
	/// Circle
	/// </summary>
	public class Circle : FillablePrimitive
	{
		//-----------------------------------//
		protected float m_Radius;            // distance from points[0] to points[1]
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public Circle() : base()
		{

		}

		//-------------------------------------------------------------------------//
		public Circle(Circle p) : base(p)
		{
			m_Radius = p.m_Radius;
		}

		//-------------------------------------------------------------------------//
		// When point 2 is added, calculate and save a radius
		protected override void Point2Added()
		{
			m_Radius = (m_Points[0] - m_Points[1]).Length(); 
		}

		//-------------------------------------------------------------------------//
		// Collider is a single ColliderCircle with our location and radius.
		public override ColliderList GetColliders()
		{
			if (!Defined) return null;

			return new ColliderList(new ColliderCircle(m_Points[0], m_Radius));
		}

		//-------------------------------------------------------------------------//
		// Geometry for a unit circle centered at (0,0).
		public override Geometry GetGeometry()
		{
			return Geometry.CircleGeometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return new Transform(new Vector3(m_Radius, m_Radius, 1.0f), new Vector3(0, 0, 0), m_Points[0]);
		}
	}
}
