using System;
using JPGL.Common.Colliders;
using JPGL.Math;

namespace JPGL.Common.Primitives
{
	/// <summary>
	/// Summary description for Line3d.
	/// </summary>
	public class Line3d : ShadablePrimitive
	{
		//-----------------------------------// 
		private DynamicGeometry m_Geometry;
		//-----------------------------------// 

		//-----------------------------------// 
		public override bool Defined         // Line is defined when it
		{                                    // has 2 vertices.
			get
			{
				return (m_Geometry.VertexCount >= 2);
			}
		}
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public Line3d() : base()
		{
			m_Geometry = new DynamicGeometry(PrimitiveRenderer.LINES);
			m_Geometry.Vertices = new float[6];
			m_Geometry.Normals  = new float[6];
			m_Geometry.Texels   = new float[4];

		}

		//-------------------------------------------------------------------------//
		public void SetEnds(float x1, float y1, float z1, float x2, float y2, float z2)
		{
			m_Geometry.Vertices[0] = x1;
			m_Geometry.Vertices[1] = y1;
			m_Geometry.Vertices[2] = z1;

			m_Geometry.Vertices[3] = x2;
			m_Geometry.Vertices[4] = y2;
			m_Geometry.Vertices[5] = z2;
		}

		//-------------------------------------------------------------------------//
		// Add the point to the Geometry
		protected override void Point1Added()
		{
			m_Geometry.Vertices[0] = m_Points[0].X;
			m_Geometry.Vertices[1] = m_Points[0].Y;
			m_Geometry.Vertices[2] = m_Points[0].Z;
		}

		//-------------------------------------------------------------------------//
		// When point 2 is added, calculate and save m_Halves segment
		protected override void Point2Added()
		{
			m_Geometry.Vertices[3] = m_Points[1].X;
			m_Geometry.Vertices[4] = m_Points[1].Y;
			m_Geometry.Vertices[5] = m_Points[1].Z;
		}

		//-------------------------------------------------------------------------//
		public override Geometry GetGeometry()
		{
			return m_Geometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return new Transform();
		}

		//-------------------------------------------------------------------------//
		public override ColliderList GetColliders()
		{
			return new ColliderList();
		}

	}
}
