using System;
using JPGL.Math;

namespace JPGL.Common.Primitives
{
	/// <summary>
	/// Summary description for LoadedPrimitive.
	/// </summary>
	public class LoadedPrimitive : Circle
	{
		//-----------------------------------//
		private int m_GeometryIndex = -1;
		public int GeometryIndex
		{
			get { return m_GeometryIndex;  }
			set { m_GeometryIndex = value; }
		}
		//-----------------------------------//

		//-----------------------------------// 
		public override bool Defined         // Defined if created
		{
			get
			{
				return (m_GeometryIndex > -1);
			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public LoadedPrimitive(string fileName) : base()
		{
			m_GeometryIndex = Geometry.LoadOBJ(fileName);

			Color = System.Drawing.Color.White;
		}

		//-------------------------------------------------------------------------//
		public LoadedPrimitive(int geometryId) : base()
		{
			m_GeometryIndex = geometryId;

			Color = System.Drawing.Color.White;
		}

		//-------------------------------------------------------------------------//
		public override Geometry GetGeometry()
		{
			return Geometry.GetLoadedGeometry(m_GeometryIndex);
		}

		//-------------------------------------------------------------------------//
		public override JPGL.Common.Colliders.ColliderList GetColliders()
		{
			JPGL.Common.Colliders.ColliderList result = new JPGL.Common.Colliders.ColliderList();
			result.AddCollider(new JPGL.Common.Colliders.ColliderSphere(new Vector3(0, 0, 0), 1));

			return result;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return new Transform(new JPGL.Math.Vector3(1, 1, 1), new Vector3(0, 0, 0), new Vector3(0, 0, 0));
		}
	}
}
