using System;
using JPGL.Common.Colliders;
using JPGL.Math;


namespace JPGL.Common.Primitives
{
	/// <summary>
	/// A Rectangle is a fillable, shadable, 2 point drawable primitive,
	/// that is defined by a center point, and vector from that center
	/// point to one corner. 
	/// </summary>
	public class Rectangle : FillablePrimitive
	{
		//-----------------------------------// The vector from point[0] (the center
		protected Vector3 m_HypBy2;          // of the rectangle) to one corner (it
		//-----------------------------------// could be any corner).

		//-----------------------------------// 
		public override bool Defined         // Rectangle is defined when the
		{                                    // midline is defined.
			get
			{
				return (m_HypBy2 != null);
			}
		}
		//-----------------------------------//

		
		//-------------------------------------------------------------------------//
		public Rectangle() : base()
		{

		}

		//-------------------------------------------------------------------------//
		public Rectangle(Rectangle p) : base(p)
		{
			m_HypBy2 = new Vector3(p.m_HypBy2);
		}


		//-------------------------------------------------------------------------//
		// When point 2 is added, calculate and save the vector to the corner.
		protected override void Point2Added()
		{
			m_HypBy2 = (m_Points[1] - m_Points[0]); 
		}

		//-------------------------------------------------------------------------//
		public override ColliderList GetColliders()
		{
			if (!Defined) return null;

			ColliderList c = new ColliderList();
			if (System.Math.Abs(m_HypBy2.X) > System.Math.Abs(m_HypBy2.Y))
			{
				float radius = System.Math.Abs(m_HypBy2.X) / 2;

				c.AddCollider(new ColliderCircle(new Vector3(m_Points[0].X+(m_HypBy2.X/2.0f), m_Points[0].Y, 0.0f), radius));
				c.AddCollider(new ColliderCircle(new Vector3(m_Points[0].X-(m_HypBy2.X/2.0f), m_Points[0].Y, 0.0f), radius));
			}
			else
			{
				float radius = System.Math.Abs(m_HypBy2.Y) / 2;

				c.AddCollider(new ColliderCircle(new Vector3(m_Points[0].X, m_Points[0].Y+(m_HypBy2.Y/2.0f), 0.0f), radius));
				c.AddCollider(new ColliderCircle(new Vector3(m_Points[0].X, m_Points[0].Y-(m_HypBy2.Y/2.0f), 0.0f), radius));
			}
			return c;
		}

		//-------------------------------------------------------------------------//
		// Square of side length 1, centered at origin.
		public override Geometry GetGeometry()
		{
			return Geometry.SquareGeometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			return new Transform(new Vector3(m_HypBy2.X*2.0f, m_HypBy2.Y*2.0f, 1.0f), new Vector3(0, 0, 0), m_Points[0]);
		}
	}
}
