using System;
using JPGL.Common.Colliders;
using JPGL.Math;


namespace JPGL.Common.Primitives
{
	/// <summary>
	/// A triangle is a fillable, shadable, 2 point drawable primitive
	/// defined by a corner and the vector from that point to the opposite
	/// face (and normal to the opposite side). Note that it is an equilateral
	/// triangle, as all the sides are the same length. A Triangle's collider
	/// is a circle collider located at the center point of the triangle, with
	/// radius equal to half the length of a side.
	/// </summary>
	public class Triangle : FillablePrimitive
	{
		//-----------------------------------//
		protected Vector3 m_MidLine;         // Vector from 1 corner ortho to opposite
		//-----------------------------------// side of the triangle.

		//-----------------------------------// 
		public override bool Defined         // Traingle is defined when the
		{                                    // midline is defined.
			get
			{
				return (m_MidLine != null);
			}
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public Triangle() : base()
		{

		}

		//-------------------------------------------------------------------------//
		public Triangle(Triangle p) : base(p)
		{
			m_MidLine = new Vector3(p.m_MidLine);
		}

		//-------------------------------------------------------------------------//
		// When point 2 is added, calculate and save a midline
		protected override void Point2Added()
		{
			m_MidLine = (m_Points[0] - m_Points[1]); 
		}

		//-------------------------------------------------------------------------//
		public override ColliderList GetColliders()
		{
			if (!Defined) return null;

			float sideLen = m_MidLine.Length() / (float)System.Math.Sin(60*(System.Math.PI/180));
			return new ColliderList(new ColliderCircle(m_Points[0]-(m_MidLine * 0.6f), sideLen / 2.0f));
		}

		//-------------------------------------------------------------------------//
		// Centered at origin with side length of 1
		public override Geometry GetGeometry()
		{
			return Geometry.TriangleGeometry;
		}

		//-------------------------------------------------------------------------//
		public override Transform GetTransform()
		{
			float sidelen = (m_MidLine.Length() * 2.0f) / (float)System.Math.Sqrt(3);
			return new Transform(new Vector3(sidelen, sidelen, 1.0f), new Vector3(0, 0, 0), m_Points[0] +  (m_MidLine * 0.5f));
		}
	}
}
