using System;

namespace JPGL.Common
{
	/// <summary>
	/// Summary description for Texture.
	/// </summary>
	public class Texture
	{
		//-----------------------------------//
		private int m_TextureIndex;
		//-----------------------------------//

		//-----------------------------------// Since Texture Transformation
		private float m_Rotation;            // happens in 2d space, we are 
		private float m_ScaleX;              // not going to go all out with 
		private float m_ScaleY;              // a full blown Transform here.
		private float m_TransX;              // We don't need all that over- 
		private float m_TransY;              // head, aka "power".
		//-----------------------------------//

		//-----------------------------------//
		private System.Drawing.Color m_BorderColor;
		//-----------------------------------//

		//-----------------------------------//
		private FilterType m_MinFilter;
		private FilterType m_MagFilter;
		//-----------------------------------//
		public enum FilterType
		{
			FILTER_TYPE_NEAREST,
			FILTER_TYPE_LINEAR,
			FILTER_TYPE_NEAREST_MIPMAP_NEAREST,
			FILTER_TYPE_LINEAR_MIPMAP_NEAREST,
			FILTER_TYPE_NEAREST_MIPMAP_LINEAR,
			FILTER_TYPE_LINEAR_MIPMAP_LINEAR
		};
		//-----------------------------------//

		//-----------------------------------//
		private AddressingMethod m_AddressingU;
		private AddressingMethod m_AddressingV;
		//-----------------------------------//
		public enum AddressingMethod
		{
			ADDRESSING_METHOD_REPEAT,
			ADDRESSING_METHOD_CLAMP,
			ADDRESSING_METHOD_MIRRORED_REPEAT,
			ADDRESSING_METHOD_CLAMP_TO_BORDER,
			ADDRESSING_METHOD_CLAMP_TO_EDGE
		};
		//-----------------------------------//

		#region Public Accessor Properties
		//-----------------------------------//
		public AddressingMethod AddressingU
		{
			get { return m_AddressingU;  }
			set { m_AddressingU = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public AddressingMethod AddressingV
		{
			get { return m_AddressingV;  }
			set { m_AddressingV = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public FilterType MinFilter
		{
			get { return m_MinFilter;  }
			set { m_MinFilter = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public FilterType MagFilter
		{
			get { return m_MagFilter;  }
			set { m_MagFilter = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public int Index
		{
			get { return m_TextureIndex;  }
			set { m_TextureIndex = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public float Rotation
		{
			get { return m_Rotation;  }
			set { m_Rotation = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public float ScaleX
		{
			get { return m_ScaleX;  }
			set { m_ScaleX = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public float ScaleY
		{
			get { return m_ScaleY;  }
			set { m_ScaleY = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public float TransX
		{
			get { return m_TransX;  }
			set { m_TransX = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public float TransY
		{
			get { return m_TransY;  }
			set { m_TransY = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public System.Drawing.Color BorderColor
		{
			get { return m_BorderColor;  }
			set { m_BorderColor = value; }
		}
		//-----------------------------------//
		#endregion

		//-------------------------------------------------------------------------//
		public Texture(int index)
		{
			m_TextureIndex = index;
			m_BorderColor  = System.Drawing.Color.FromArgb(255, 255, 255, 255);
			m_Rotation = 0;
			m_ScaleX   = 1;
			m_ScaleY   = 1;
			m_TransX   = 0;
			m_TransY   = 0;

			m_MinFilter = FilterType.FILTER_TYPE_LINEAR;
			m_MagFilter = FilterType.FILTER_TYPE_LINEAR;

			m_AddressingU = AddressingMethod.ADDRESSING_METHOD_CLAMP;
			m_AddressingV = AddressingMethod.ADDRESSING_METHOD_CLAMP;
			
		}

		//-------------------------------------------------------------------------//
		public Texture(Texture old)
		{
			this.m_TextureIndex = old.m_TextureIndex;
			this.m_BorderColor  = old.m_BorderColor;
			this.m_Rotation = old.m_Rotation;
			this.m_ScaleX   = old.m_ScaleX;
			this.m_ScaleY   = old.m_ScaleY;
			this.m_TransX   = old.m_TransX;
			this.m_TransY   = old.m_TransY;

			this.m_MinFilter = old.m_MinFilter;
			this.m_MagFilter = old.m_MagFilter;

			this.m_AddressingU = old.m_AddressingU;
			this.m_AddressingV = old.m_AddressingV;
		}

		//-------------------------------------------------------------------------//
		public void SetTextureTiled()
		{
			m_MinFilter = FilterType.FILTER_TYPE_NEAREST;
			m_MagFilter = FilterType.FILTER_TYPE_NEAREST;
			m_AddressingU = AddressingMethod.ADDRESSING_METHOD_CLAMP_TO_EDGE;
			m_AddressingV = AddressingMethod.ADDRESSING_METHOD_CLAMP_TO_EDGE;
		}
	}
}
