using System;
using JPGL.Math;

namespace JPGL.Common
{
	/// <summary>
	/// Summary description for Transform.
	/// </summary>
	public class Transform
	{
		//-----------------------------------//
		private Vector3    m_Translation;    // Translation
		private Vector3    m_Scale;          // Scaling
		private Quaternion m_Rotation;       // Rotation
		private Vector3    m_Pivot;          // Pivot (by default, P=T)
		//-----------------------------------//

		//-----------------------------------//
		private static Vector3 XAxis = new Vector3(1, 0, 0);
		private static Vector3 YAxis = new Vector3(0, 1, 0);
		private static Vector3 ZAxis = new Vector3(0, 0, 1);
		//-----------------------------------//

		#region Accessor Properties
		//-----------------------------------//
		public Vector3 S
		{
			get { return m_Scale;  }
			set { m_Scale = value; }
		}
		public Vector3 R
		{
			set
			{
				m_Rotation = new Quaternion();
				m_Rotation *= new Quaternion(XAxis, value.X);
				m_Rotation *= new Quaternion(XAxis, value.X);
				m_Rotation *= new Quaternion(XAxis, value.X);
			}
		}
		public float RXBy
		{
			set { m_Rotation *= new Quaternion(XAxis, value); }
		}
		public float RYBy
		{
			set { m_Rotation *= new Quaternion(YAxis, value); }
		}
		public float RZBy
		{
			set { m_Rotation *= new Quaternion(ZAxis, value); }
		}
		public Quaternion RQ
		{
			get { return m_Rotation;  }
			set { m_Rotation = value; }
		}
		public Vector3 P
		{
			get { return m_Pivot;  }
			set { m_Pivot = value; }
		}
		public Vector3 T
		{
			get { return m_Translation;  }
			set { m_Translation = value; }
		}
		//-----------------------------------//
		#endregion


		//-------------------------------------------------------------------------//
		public Transform()
		{
			m_Translation    = new Vector3(0, 0, 0);
			m_Scale          = new Vector3(1, 1, 1);
			m_Rotation       = new Quaternion();
			m_Pivot          = new Vector3(0, 0, 0);
		}

		//-------------------------------------------------------------------------//
		public Transform(Transform old)
		{
			m_Scale          = new Vector3(old.m_Scale);
			m_Rotation       = new Quaternion(old.m_Rotation);
			m_Pivot          = new Vector3(old.m_Pivot);
			m_Translation    = new Vector3(old.m_Translation);
		}

		//-------------------------------------------------------------------------//
		public Transform(Vector3 s, Vector3 r, Vector3 t)
		{
			m_Scale          = s;
			m_Rotation       = new Quaternion();
			m_Rotation       *= new Quaternion(XAxis, r.X);
			m_Rotation       *= new Quaternion(YAxis, r.Y);
			m_Rotation       *= new Quaternion(ZAxis, r.Z);
			m_Pivot          = new Vector3(0, 0, 0);
			m_Translation    = t;
		}
	}
}
