using System;

namespace JPGL.Math
{
	/// <summary>
	/// A 4x4 Matrix
	/// </summary>
	public class Matrix
	{
		//-----------------------------------//
		internal Sharp3D.Math.Core.Matrix4F m_Mat;
		//-----------------------------------//


		//-----------------------------------//
		public float this[int r, int c]
		{
			get { return m_Mat[r+1, c+1];  }
			set { m_Mat[r+1, c+1] = value;}
		}
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public Matrix()
		{
			m_Mat = Sharp3D.Math.Core.Matrix4F.Identity;
		}

		//-------------------------------------------------------------------------//
		public Matrix(Matrix old)
		{
			m_Mat = new Sharp3D.Math.Core.Matrix4F(old.m_Mat);
		}

		//-------------------------------------------------------------------------//
		public Matrix(
			float m11, float m12, float m13, float m14, 
			float m21, float m22, float m23, float m24, 
			float m31, float m32, float m33, float m34, 
			float m41, float m42, float m43, float m44)
		{
			m_Mat = new Sharp3D.Math.Core.Matrix4F(m11, m12, m13, m14, m21, m22, m23, m24, m31, m32, m33, m34, m41, m42, m43, m44);
		}

		//-------------------------------------------------------------------------//
		// Transform the given point
		public Vector3 Transform(Vector3 point)
		{
			Sharp3D.Math.Core.Vector4F result = Sharp3D.Math.Core.Matrix4F.Transform(m_Mat, new Sharp3D.Math.Core.Vector4F(point.X, point.Y, point.Z, 0));
			return new Vector3(result.X, result.Y, result.Z);
		}

		//-------------------------------------------------------------------------//
		// Return a new matrix which is the transpose of this matrix
		public Matrix Transpose()
		{
			Matrix result = new Matrix(this);
			result.m_Mat.Transpose();
			return result;
		}

	}
}
