using System;

namespace JPGL.Math
{
	/// <summary>
	/// Evil 4D math to the rotational rescue.
	/// </summary>
	public class Quaternion
	{
		//-----------------------------------//
		internal Sharp3D.Math.Core.QuaternionF m_Quat;
		//-----------------------------------//

		//-----------------------------------//
		public float X
		{
			get { return m_Quat.X;  }
			set { m_Quat.X = value; }
		}
		public float Y
		{
			get { return m_Quat.Y;  }
			set { m_Quat.Y = value; }
		}
		public float Z
		{
			get { return m_Quat.Z;  }
			set { m_Quat.Z = value; }
		}
		public float W
		{
			get { return m_Quat.W;  }
			set { m_Quat.W = value; }
		}
		//-----------------------------------//

		//-----------------------------------//
		public Quaternion Conjugate
		{
			get
			{
				Quaternion result = new Quaternion(new Vector3(), 0);
				result.m_Quat = this.m_Quat.GetConjugate();
				return result;
			}
		}
		//-----------------------------------//

		//-----------------------------------//
		public Quaternion Inverse
		{
			get
			{
				Quaternion result = new Quaternion(this);
				result.m_Quat.Inverse();
				return result;
			}
		}

		//-------------------------------------------------------------------------//
		public Quaternion()
		{
			m_Quat = Sharp3D.Math.Core.QuaternionF.Identity;
		}

		//-------------------------------------------------------------------------//
		public Quaternion(Quaternion old)
		{
			m_Quat = new Sharp3D.Math.Core.QuaternionF(old.m_Quat);

		}

		//-------------------------------------------------------------------------//
		public Quaternion(Vector3 axis, float degreeRotation)
		{
			float angle = (float)((degreeRotation / 180.0f) * System.Math.PI);
			float result = (float)System.Math.Sin( angle / 2.0f );

			m_Quat = new Sharp3D.Math.Core.QuaternionF(
						(float)System.Math.Cos( angle / 2.0f ),
						(float)(axis.X * result),
						(float)(axis.Y * result),
						(float)(axis.Z * result));
		}

		//-------------------------------------------------------------------------//
		public Quaternion(float w, float x, float y, float z)
		{
			m_Quat = new Sharp3D.Math.Core.QuaternionF(w, x, y, z);
		}

		//-------------------------------------------------------------------------//
		public Quaternion(Matrix m)
		{
			m_Quat = Sharp3D.Math.Core.QuaternionF.Identity;

			float t = m[0,0] + m[1,1] + m[2,2] + 1;
			if ( t > 0.0001 ) 
			{
				float s  = (float)System.Math.Sqrt(t) * 2.0f;
				m_Quat.W = (0.25f * s);
				m_Quat.X = (m[1,2] - m[2,1]) / s;
				m_Quat.Y = (m[2,0] - m[0,2]) / s;
				m_Quat.Z = (m[0,1] - m[1,0]) / s;
			}
			else 
			{
				if ( m[0,0] > m[1,1] && m[0,0] > m[2,2] ) {
					float s = 2.0f * (float)System.Math.Sqrt( 1.0f + m[0,0] - m[1,1] - m[2,2]);
					m_Quat.X = (float)(0.25 * s);
					m_Quat.Y = (m[0,1] + m[1,0] ) / s;
					m_Quat.Z = (m[0,2] + m[2,0] ) / s;
					m_Quat.W = (m[1,2] - m[2,1] ) / s;
				} else if (m[1,1] > m[2,2]) {
					float s = 2.0f * (float)System.Math.Sqrt( 1.0f + m[1,1] - m[0,0] - m[2,2]);
					m_Quat.X = (m[0,1] + m[1,0] ) / s;
					m_Quat.Y = (float)(0.25 * s);
					m_Quat.Z = (m[1,2] + m[2,1] ) / s;
					m_Quat.W = (m[0,2] - m[2,0] ) / s;
				} else {
					float s = 2.0f * (float)System.Math.Sqrt( 1.0f + m[2,2] - m[0,0] - m[1,1] );
					m_Quat.X = (m[0,2] + m[2,0] ) / s;
					m_Quat.Y = (m[1,2] + m[2,1] ) / s;
					m_Quat.Z = (float)(0.25 * s);
					m_Quat.W = (m[0,1] - m[1,0] ) / s;
				}
			}

		}

		//-------------------------------------------------------------------------//
		public float[] GetMatrix()
		{
			float[] result = new float[16];

			result[ 0] = 1.0f - 2.0f * (m_Quat.Y * m_Quat.Y + m_Quat.Z * m_Quat.Z); 
			result[ 1] = 2.0f * (m_Quat.X * m_Quat.Y + m_Quat.Z * m_Quat.W);
			result[ 2] = 2.0f * (m_Quat.X * m_Quat.Z - m_Quat.Y * m_Quat.W);
			result[ 3] = 0.0f;  

			result[ 4] = 2.0f * (m_Quat.X * m_Quat.Y - m_Quat.Z * m_Quat.W);  
			result[ 5] = 1.0f - 2.0f * (m_Quat.X * m_Quat.X + m_Quat.Z * m_Quat.Z); 
			result[ 6] = 2.0f * (m_Quat.Z * m_Quat.Y + m_Quat.X * m_Quat.W );  
			result[ 7] = 0.0f;  

			result[ 8] = 2.0f * (m_Quat.X * m_Quat.Z + m_Quat.Y * m_Quat.W);
			result[ 9] = 2.0f * (m_Quat.Y * m_Quat.Z - m_Quat.X * m_Quat.W);
			result[10] = 1.0f - 2.0f * (m_Quat.X * m_Quat.X + m_Quat.Y * m_Quat.Y);  
			result[11] = 0.0f;  

			result[12] = 0;  
			result[13] = 0;  
			result[14] = 0;  
			result[15] = 1.0f;

			return result;
		}

		//-------------------------------------------------------------------------//
		public Matrix GetMat()
		{
			float[] result = GetMatrix();
			return new Matrix(
				result[0], result[1], result[2], result[3],
				result[4], result[5], result[6], result[7],
				result[8], result[9], result[10], result[11],
				result[12], result[13], result[14], result[15]);

		}

		//-------------------------------------------------------------------------//
		public static Quaternion operator *(Quaternion q1, Quaternion q2)
		{
			Quaternion result = new Quaternion(new Vector3(0, 0, 0), 0);
			result.m_Quat = Sharp3D.Math.Core.QuaternionF.Multiply(q1.m_Quat, q2.m_Quat);
			return result;
		}

		//-------------------------------------------------------------------------//
		public Quaternion Slerp(Quaternion other, float t)
		{
			Quaternion ret = new Quaternion();

			// slerp(q1,q2) = sin((1-t)*a)/sin(a) * q1  +  sin(t*a)/sin(a) * q2
			Sharp3D.Math.Core.QuaternionF q1 = new Sharp3D.Math.Core.QuaternionF(this.m_Quat);
			Sharp3D.Math.Core.QuaternionF q2 = new Sharp3D.Math.Core.QuaternionF(other.m_Quat);
			q1.Normalize();
			q2.Normalize();
	
			float dot = Sharp3D.Math.Core.QuaternionF.DotProduct(q1, q2);
			if (dot > 0.9995f)
			{
				ret.m_Quat = q1 + t * (q2 - q1);
				ret.m_Quat.Normalize();
				return ret;
			}

			float a = (float)System.Math.Acos(dot);
			float s = (float)System.Math.Sin(a);

			ret.m_Quat = (float)System.Math.Sin((1.0f-t)*a)/s * q1 + 
						 (float)System.Math.Sin(t*a)/s * q2;
			ret.m_Quat.Normalize();
			return ret;
		}

	}
}
