using System;

namespace JPGL.Math
{
	/// <summary>
	/// Vector3 is a wrapper for however vector3 is implemented.
	/// </summary>
	public class Vector3
	{
		internal Sharp3D.Math.Core.Vector3F m_Vec3;

		// Public interface to vector components
		public float X
		{
			get { return m_Vec3.X;  }
			set { m_Vec3.X = value; }
		}
		public float Y
		{
			get { return m_Vec3.Y;  }
			set { m_Vec3.Y = value; }
		}
		public float Z
		{
			get { return m_Vec3.Z;  }
			set { m_Vec3.Z = value; }
		}

		//
		// Default constructor
		//
		public Vector3()
		{
			m_Vec3 = new Sharp3D.Math.Core.Vector3F(0, 0, 0);
		}

		//
		// Initializing constructor
		//
		public Vector3(float x, float y, float z)
		{
			m_Vec3 = new Sharp3D.Math.Core.Vector3F(x, y, z);
		}

		//
		// Copy constructor
		//
		public Vector3(JPGL.Math.Vector3 old)
		{
			m_Vec3 = new Sharp3D.Math.Core.Vector3F(old.m_Vec3);
		}

		//
		// Private copy constructor, unfortunately c# ignores this private, so
		// the addition of another argument is necessary. There really must be
		// some better way to do this.
		//
		private Vector3(Sharp3D.Math.Core.Vector3F v, int dumbhack)
		{
			m_Vec3 = new Sharp3D.Math.Core.Vector3F(v);
		}

		//
		// Length of this vector
		//
		public float Length()
		{
			return m_Vec3.GetLength();
		}

		//
		// return normalized version of this vector
		//
		public Vector3 Normalize()
		{
			Sharp3D.Math.Core.Vector3F v = new Sharp3D.Math.Core.Vector3F(m_Vec3);
			try
			{
				v.Normalize();
			}
			catch
			{
				return new Vector3(new Sharp3D.Math.Core.Vector3F(m_Vec3), 0);
			}
			return new Vector3(v, 0);
		}

		//
		// Return a new vector, which is a projection of this vector
		// onto the given vector
		//
		public Vector3 ProjectOnto(Vector3 a)
		{
			Vector3 result = a * ((Sharp3D.Math.Core.Vector3F.DotProduct(a.m_Vec3, this.m_Vec3)) / a.m_Vec3.GetLengthSquared());
			return result;
		}

		//
		// Get a vector in the same 2d plane (z=0) as this vector
		// , that is orthogonal to this vector.
		//
		public Vector3 OrthoVector2d()
		{
			Vector3 normal = new Vector3(0, 0, 1);
			return (new Vector3(Sharp3D.Math.Core.Vector3F.CrossProduct(m_Vec3, normal.m_Vec3), 0));
		}

		//
		// Cross product
		//
		public Vector3 CrossProduct(Vector3 other)
		{
			return new Vector3(Sharp3D.Math.Core.Vector3F.CrossProduct(this.m_Vec3, other.m_Vec3), 0);
		}

		//
		// Scalar multiplication
		//
		public static Vector3 operator *(Vector3 v, float scale)
		{
			return new Vector3(v.X*scale, v.Y*scale, v.Z*scale);
		}

		//
		// Scalar multiplication
		//
		public static Vector3 operator *(float scale, Vector3 v)
		{
			return new Vector3(v.X*scale, v.Y*scale, v.Z*scale);
		}

		//
		// Dot product
		//
		public static float operator *(Vector3 v1, Vector3 v2)
		{
			return (v1.X*v2.X + v1.Y*v2.Y + v1.Z*v2.Z);
		}

		//
		// Vector subtraction
		//
		public static Vector3 operator -(Vector3 v1, Vector3 v2)
		{
			return new Vector3(v1.X-v2.X , v1.Y-v2.Y, v1.Z-v2.Z);
		}

		//
		// Vector addition
		//
		public static Vector3 operator +(Vector3 v1, Vector3 v2)
		{
			return new Vector3(v1.X+v2.X , v1.Y+v2.Y, v1.Z+v2.Z);
		}

		//
		// Matrix multiplication
		//
		public static Vector3 operator *(Matrix m, Vector3 v)
		{
			Sharp3D.Math.Core.Vector4F result =  m.m_Mat * new Sharp3D.Math.Core.Vector4F(v.m_Vec3.X, v.m_Vec3.Y, v.m_Vec3.Z, 1);

			return new Vector3(result.X, result.Y, result.Z);
		}

		public static Vector3 operator *(Vector3 v, Matrix m)
		{
			Sharp3D.Math.Core.Matrix4F mt = m.m_Mat;
			mt.Transpose();
			Sharp3D.Math.Core.Vector4F result =  mt * new Sharp3D.Math.Core.Vector4F(v.m_Vec3.X, v.m_Vec3.Y, v.m_Vec3.Z, 1);

			return new Vector3(result.X, result.Y, result.Z);
		}

	}
}
