using System;
using System.Collections;

namespace JPGL.Math
{
	/// <summary>
	/// Summary description for Vector3List.
	/// </summary>
	public class Vector3List : IEnumerable
	{
		//-----------------------------------//
		private ArrayList m_Vectors;         //
		//-----------------------------------//
		public Vector3 this [int index]
		{
			get
			{
				if (m_Vectors.Count <= index) return null;
				return (Vector3)m_Vectors[index];
			}
			set	
			{ 
				m_Vectors.Insert(index, value); 
			}
		}
		//-----------------------------------//
		public int Count
		{
			get { return m_Vectors.Count; }
		}
		//-----------------------------------//

		//-------------------------------------------------------------------------//
		public Vector3List()
		{
			m_Vectors = new ArrayList();
		}

		//-------------------------------------------------------------------------//
		public Vector3List(Vector3List old)
		{
			m_Vectors = new ArrayList();
			foreach (Vector3 v in old)
			{
				this.Add(new Vector3(v));
			}
		}

		//-------------------------------------------------------------------------//
		// Add a vector to the list
		public void Add(Vector3 p)
		{
			m_Vectors.Add(p);
		}

		//-------------------------------------------------------------------------//
		// Remove the given vector from the list
		public void Remove(Vector3 p)
		{
			m_Vectors.Remove(p);
		}

		public void Insert(int index, Vector3 v)
		{
			m_Vectors.Insert(index, v);
		}

		//-------------------------------------------------------------------------//
		// Enumeration functions (so we can use foreach ... in Vector3List)
		public IEnumerator GetEnumerator()
		{
			return new Vector3ListEnumerator(this);
		} 

		/// <summary>
		/// Private Enumerator so we can use foreach on our lists.
		/// </summary>
		private class Vector3ListEnumerator : IEnumerator
		{
			//-----------------------------------//
			private Vector3List m_List;          // Reference to list
			private int         m_Position;      // Position used for enumeration
			//-----------------------------------//

			public Vector3ListEnumerator(Vector3List list)
			{
				m_List     = list;
				m_Position = -1;
			}

			//-------------------------------------------------------------------------//
			public bool MoveNext()
			{
				m_Position++;
				if (m_Position >= m_List.m_Vectors.Count)
				{
					return false;
				}

				return true;
			} 

			//-------------------------------------------------------------------------//
			public void Reset()
			{
				m_Position = -1;
			} 

			//-------------------------------------------------------------------------//
			public object Current
			{
				get
				{
					return (m_List.m_Vectors[m_Position]);
				}
			}
		}
	}
}
