using System;
using System.ComponentModel;
using System.Windows.Forms;


namespace JPGL.SWF
{
	/// <summary>
	/// SliderWithEcho.
	/// </summary>
	public class SliderWithEcho : System.Windows.Forms.UserControl
	{
		//-----------------------------------// Slider Event handler
		public event SliderChangeHandler SliderChange;
		protected virtual void OnSliderChange(SliderEventArgs e)
		{
			if (SliderChange != null)
				SliderChange(this, e);
		}
		//-----------------------------------//

		//-----------------------------------//
		private float m_Slope;               // Slope of linear mapping
		private float m_Offset;              // Offset of linear mapping
		//-----------------------------------//

		//-----------------------------------//
		public float Value                   // Get/Set the mapped value of the slider
		{
			get { return (Slider_1.Value*m_Slope) + m_Offset; }
			set 
			{ 
				Slider_1.Value = (int)((value-m_Offset)/m_Slope);
				Echo_1.Text    = String.Format("{0:f}", value);
			}
		}
		//-----------------------------------//

		//-----------------------------------//
		public System.Drawing.Color LabelTextColor
		{
			set { Text_Label.BackColor = System.Drawing.SystemColors.Control; Text_Label.ForeColor = value; }
		}
		//-----------------------------------//

		private System.Windows.Forms.TextBox  Echo_1;
		private System.Windows.Forms.Panel    panel2;
		private System.Windows.Forms.TrackBar Slider_1;
		private System.Windows.Forms.TextBox  Text_Label;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		//-------------------------------------------------------------------------//
		public SliderWithEcho()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			m_Slope  = 1;
			m_Offset = 0;
		}

		//-------------------------------------------------------------------------//
		// Setup the linear mapping and initial slider values
		public void SetupSlider(string label, int resolution, float initial, float min, float max)
		{
			Enabled = true;
			Text_Label.Text = label;

			m_Slope  = (max - min) / resolution;
			m_Offset = min;
	
			Slider_1.Minimum = 0;		
			Slider_1.Maximum = resolution;
			
			Slider_1.Value   = (int)((initial-m_Offset)/m_Slope);
			Echo_1.Text      = String.Format("{0:f}", initial);
		}


		//-------------------------------------------------------------------------//
		// Slider scroll event handler. Update the echo box, and let the registered
		// event handlers know about the event.
		private void Slider_1_Scroll(object sender, System.EventArgs e)
		{
			Echo_1.Text = String.Format("{0:f}", (Slider_1.Value*m_Slope) + m_Offset);

			OnSliderChange(new SliderEventArgs((Slider_1.Value*m_Slope) + m_Offset));
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Echo_1 = new System.Windows.Forms.TextBox();
			this.panel2 = new System.Windows.Forms.Panel();
			this.Text_Label = new System.Windows.Forms.TextBox();
			this.Slider_1 = new System.Windows.Forms.TrackBar();
			this.panel2.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.Slider_1)).BeginInit();
			this.SuspendLayout();
			// 
			// Echo_1
			// 
			this.Echo_1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.Echo_1.AutoSize = false;
			this.Echo_1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Echo_1.Cursor = System.Windows.Forms.Cursors.Default;
			this.Echo_1.Location = new System.Drawing.Point(233, 3);
			this.Echo_1.Name = "Echo_1";
			this.Echo_1.ReadOnly = true;
			this.Echo_1.Size = new System.Drawing.Size(43, 20);
			this.Echo_1.TabIndex = 1;
			this.Echo_1.TabStop = false;
			this.Echo_1.Text = "";
			this.Echo_1.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			// 
			// panel2
			// 
			this.panel2.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel2.BackColor = System.Drawing.Color.SteelBlue;
			this.panel2.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.panel2.Controls.Add(this.Text_Label);
			this.panel2.Controls.Add(this.Slider_1);
			this.panel2.Controls.Add(this.Echo_1);
			this.panel2.Location = new System.Drawing.Point(0, 0);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(285, 51);
			this.panel2.TabIndex = 3;
			// 
			// Text_Label
			// 
			this.Text_Label.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.Text_Label.AutoSize = false;
			this.Text_Label.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Text_Label.Cursor = System.Windows.Forms.Cursors.Default;
			this.Text_Label.Location = new System.Drawing.Point(5, 3);
			this.Text_Label.Name = "Text_Label";
			this.Text_Label.ReadOnly = true;
			this.Text_Label.Size = new System.Drawing.Size(224, 20);
			this.Text_Label.TabIndex = 3;
			this.Text_Label.TabStop = false;
			this.Text_Label.Text = "";
			// 
			// Slider_1
			// 
			this.Slider_1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.Slider_1.AutoSize = false;
			this.Slider_1.Location = new System.Drawing.Point(6, 28);
			this.Slider_1.Name = "Slider_1";
			this.Slider_1.Size = new System.Drawing.Size(271, 20);
			this.Slider_1.TabIndex = 2;
			this.Slider_1.TabStop = false;
			this.Slider_1.TickStyle = System.Windows.Forms.TickStyle.None;
			this.Slider_1.Scroll += new System.EventHandler(this.Slider_1_Scroll);
			// 
			// SliderWithEcho
			// 
			this.Controls.Add(this.panel2);
			this.Name = "SliderWithEcho";
			this.Size = new System.Drawing.Size(285, 51);
			this.panel2.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.Slider_1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

	}

	/// <summary>
	/// SliderEvent argument class.
	/// </summary>
	public class SliderEventArgs: EventArgs
	{
		//-----------------------------------//
		private float m_Value;
		public float Value
		{
			get { return m_Value; }
		}
		//-----------------------------------//


		//-------------------------------------------------------------------------//
		public SliderEventArgs(float v)
		{
			m_Value = v;
		}
	}

	//-------------------------------------------------------------------------//
	public delegate void SliderChangeHandler(object sender, SliderEventArgs e);
}
